@@ -10,8 +10,8 @@
 package org.gnu.atk;
 
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
 
 /**
  */
@@ -22,7 +22,11 @@ public class ObjectFactory extends GObject {
     }
     
     public AtkObject createAccessible(GObject object) {
-        return new AtkObject(atk_object_factory_create_accessible(getHandle(), object.getHandle()));
+        return 
+            AtkObject.
+            getAtkObjectFromHandle( 
+                atk_object_factory_create_accessible(getHandle(), 
+                                                     object.getHandle()) );
     }
     
     public void invalidate() {
