@@ -9,7 +9,7 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
 
 /**
  * 
