@@ -9,8 +9,8 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
 
 /**
  * HButtonBox is a container designed to display a collection of
@@ -36,6 +36,20 @@ public class HButtonBox extends ButtonBox {
 	public HButtonBox(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Construct a new HButtonBox from a handle to a native resource.
+     */
+    public static HButtonBox getHButtonBox(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        HButtonBox obj = (HButtonBox) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new HButtonBox(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Retrieve the runtime type used by the GLib library.
@@ -48,5 +62,11 @@ public class HButtonBox extends ButtonBox {
     native static final protected int gtk_hbutton_box_get_type ();
     native static final protected Handle gtk_hbutton_box_new ();
 
+    /* Deprectated functions.
+    native static final private int gtk_hbutton_box_get_spacing_default();
+    native static final private int gtk_hbutton_box_get_layout_default();
+    native static final private void gtk_hbutton_box_set_spacing_default(int spacing);
+    native static final private void gtk_hbutton_box_set_layout_default(int layout);
+    */
 }
 
