@@ -9,8 +9,8 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
 
 /**
  * The MenuShell class is an abstract base class used to derive the Menu
@@ -91,6 +91,35 @@ public abstract class MenuShell extends Container {
 		MenuShell.gtk_menu_shell_activate_item(getHandle(), menuItem.getHandle(), forceDeactivate);
 	}
 	
+    /**
+     * Select the first visible or selectable child of the menu shell.
+     * It won't select tearoff items unless the only item is a tearoff
+     * item.
+     *
+     * @param searchSensitive If TRUE, search for the first selectable
+     * menu item, otherwise select nothing if the first item isn't
+     * sensitive. This should be FALSE if the menu is being popped up
+     * initially.
+     */
+    public void selectFirst( boolean searchSensitive ) {
+        gtk_menu_shell_select_first( getHandle(), searchSensitive );
+    }
+    /**
+     * Cancels the selection within the menu shell.
+     */
+    public void cancel() {
+        gtk_menu_shell_cancel( getHandle() );
+    }
+
+	public boolean getTakeFocus() {
+		return gtk_menu_shell_get_take_focus(getHandle());
+	}
+	
+	public void setTakeFocus(boolean takeFocus) {
+		gtk_menu_shell_set_take_focus(getHandle(), takeFocus);
+	}
+	
+	
 	/**
 	 * Retrieve the runtime type used by the GLib library.
 	 */
@@ -107,5 +136,10 @@ public abstract class MenuShell extends Container {
 	native static final protected void gtk_menu_shell_select_item(Handle menu_shell, Handle menuItem);
 	native static final protected void gtk_menu_shell_deselect(Handle menu_shell);
 	native static final protected void gtk_menu_shell_activate_item(Handle menu_shell, Handle menuItem, boolean forceDeactivate);
+    native static final private void gtk_menu_shell_select_first(Handle menu_shell, boolean search_sensitive);
+    native static final private void gtk_menu_shell_cancel(Handle menu_shell);
+	// new for gtk 2.7
+	native static final private boolean gtk_menu_shell_get_take_focus(Handle menu_shell);
+	native static final private void gtk_menu_shell_set_take_focus(Handle menu_shell, boolean take_focus);
 
 }
