@@ -13,11 +13,10 @@ import java.util.Vector;
 
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.OptionMenuEvent;
 import org.gnu.gtk.event.OptionMenuListener;
+import org.gnu.javagnome.Handle;
 
 /**
  * deprecated menu widget replaced by {@link ComboBox} in gtk 2.3.
@@ -62,10 +61,7 @@ public class OptionMenu extends Button {
 	 */
 	public Menu getMenu() {
 	    Handle hndl = OptionMenu.gtk_option_menu_get_menu(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Menu)obj;
-		return new Menu(hndl);
+		return Menu.getMenu(hndl);
 	}
 	
 	/**
