@@ -9,8 +9,7 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
 
 /**
  */
@@ -59,6 +58,17 @@ public class RadioToolButton extends ToggleToolButton {
 	private RadioToolButton(Handle hndl) {
 		super(hndl);
 	}
+    
+    private static RadioToolButton getRadioToolButton(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        RadioToolButton obj = (RadioToolButton) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new RadioToolButton(handle);
+        
+        return obj;
+    }
 	
 	public RadioToolButton[] getGroup() {
 	    Handle[] hndls = gtk_radio_tool_button_get_group(getHandle());
@@ -66,11 +76,7 @@ public class RadioToolButton extends ToggleToolButton {
 			return null;
 		RadioToolButton[] group = new RadioToolButton[hndls.length];
 		for (int i = 0; i < hndls.length; i++) {
-			GObject obj = getGObjectFromHandle(hndls[i]);
-			if (null != obj)
-				group[i] = (RadioToolButton)obj;
-			else
-				group[i] = new RadioToolButton(hndls[i]);
+			group[i] = RadioToolButton.getRadioToolButton(hndls[i]);
 		}
 		return group;
 	}
