@@ -13,11 +13,10 @@ import java.util.Vector;
 
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.RangeEvent;
 import org.gnu.gtk.event.RangeListener;
+import org.gnu.javagnome.Handle;
 
 /**
  * This is the base class for the group of widgets that involve graphic
@@ -40,10 +39,7 @@ public abstract class Range extends Widget {
 	 */
 	public Adjustment getAdjustment() {
 	    Handle hndl = gtk_range_get_adjustment(getHandle());
-		GObject obj = retrieveGObject(hndl);
-		if (null != obj)
-			return (Adjustment)obj;
-		return new Adjustment(hndl);
+		return Adjustment.getAdjustment(hndl);
 	}
 
 	/**
