@@ -10,6 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
 
 /**
  * The ScrollBar widget is an abstract base class for {@link HScrollBar}
@@ -32,7 +33,7 @@ public abstract class ScrollBar extends Range
 	/**
 	 * Construct a ScrollBar from a handle to a native resource.
 	 */
-	protected ScrollBar(int handle) {
+	protected ScrollBar(Handle handle) {
 	    super(handle);
 	}
 
@@ -43,12 +44,6 @@ public abstract class ScrollBar extends Range
 		return new Type(gtk_scrollbar_get_type());
 	}
 
-    /****************************************
-     * BEGINNING OF GENERATED CODE
-     ****************************************/
     native static final protected int gtk_scrollbar_get_type ();
-    /****************************************
-     * END OF GENERATED CODE
-     ****************************************/
 }
 
