@@ -9,6 +9,8 @@
 
 package org.gnu.gtk;
 
+import org.gnu.javagnome.Handle;
+
 /**
  */
 public class SeparatorToolItem extends ToolItem {
@@ -17,7 +19,7 @@ public class SeparatorToolItem extends ToolItem {
 		super(gtk_separator_tool_item_new());
 	}
 	
-	public SeparatorToolItem(int hndl) {
+	public SeparatorToolItem(Handle hndl) {
 		super(hndl);
 	}
 	
@@ -33,20 +35,13 @@ public class SeparatorToolItem extends ToolItem {
 	/**
 	 * Returns whether the SeparatorToolItem is drawn as a line or
 	 * just blank.
-	 * @return
 	 */
 	public boolean getDrawLine() {
 		return gtk_separator_tool_item_get_draw(getHandle());
 	}
 
-	/****************************************
-	 * BEGINNING OF JNI CODE
-	 ****************************************/
 	native static final protected int gtk_separator_tool_item_get_type();
-	native static final protected int gtk_separator_tool_item_new();
-	native static final protected boolean gtk_separator_tool_item_get_draw(int item);
-	native static final protected void gtk_separator_tool_item_set_draw(int item, boolean draw);
-	/****************************************
-	 * END OF JNI CODE
-	 ****************************************/
+	native static final protected Handle gtk_separator_tool_item_new();
+	native static final protected boolean gtk_separator_tool_item_get_draw(Handle item);
+	native static final protected void gtk_separator_tool_item_set_draw(Handle item, boolean draw);
 }
