@@ -9,19 +9,16 @@
 
 package org.gnu.gtk;
 
-import org.gnu.glib.Boxed;
-import org.gnu.glib.Handle;
+import org.gnu.glib.MemStruct;
+import org.gnu.javagnome.Handle;
 
 /**
  * Definition for sources and destinations of Drag and Drop operations.
  *
  * If a drag and drop is allowed between a pair of widgets, they must have the
  * same TargetEntry in their drop source and destination lists. 
- *
- *
- * TODO: Free the memory !!
  */
-public class TargetEntry extends Boxed
+public class TargetEntry extends MemStruct
 {
 	/**
 	 * Constructs a new target entry
@@ -29,7 +26,7 @@ public class TargetEntry extends Boxed
 	 * the drag source and destination (possibly different applications).
 	 * @param flags 
 	 * @param id Integer id for the drag type. This will be passed to the event
-	 * handers. It can be different to that used at the other end of the
+	 * handlers. It can be different to that used at the other end of the
 	 * drag/drop operation.
 	 */
 	public TargetEntry( String name, TargetFlags flags, int id ){
