@@ -10,6 +10,7 @@
 package org.gnu.gtk;
 
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
 
 /**
  *
@@ -26,7 +27,7 @@ public class TearoffMenuItem extends MenuItem
 	/**
 	 * Construct a TearoffMenuItem using a handle to a native resource.
 	 */
-	public TearoffMenuItem(int handle) {
+	public TearoffMenuItem(Handle handle) {
 	    super(handle);
 	}
 
@@ -41,14 +42,8 @@ public class TearoffMenuItem extends MenuItem
 		return new Type(gtk_tearoff_menu_item_get_type());
 	}
 
-    /****************************************
-     * BEGINNING OF GENERATED CODE
-     ****************************************/
     native static final protected int gtk_tearoff_menu_item_get_type ();
-    native static final protected int gtk_tearoff_menu_item_new ();
-    native static final protected boolean getTornOff(int menu);
-    /****************************************
-     * END OF GENERATED CODE
-     ****************************************/
+    native static final protected Handle gtk_tearoff_menu_item_new ();
+    native static final protected boolean getTornOff(Handle menu);
 }
 
