@@ -14,10 +14,10 @@ import java.util.Vector;
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.TextBufferEvent;
 import org.gnu.gtk.event.TextBufferListener;
+import org.gnu.javagnome.Handle;
 
 /**
  * The TextBuffer is the object used to store data to be displayed in a {@link
@@ -36,10 +36,24 @@ public class TextBuffer extends GObject {
 	protected TextBuffer(Handle handle) {
 		super(handle);
 	}
+    
+    protected static TextBuffer getTextBuffer(Handle handle) {
+        if (handle == null)
+            return null;
+
+        TextBuffer obj = (TextBuffer) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new TextBuffer(handle);
+
+        return obj;
+    }
+    
 	/**
-	 * Creates a new text buffer.
-	 * @param table A tag table
-	 */
+     * Creates a new text buffer.
+     * 
+     * @param table
+     *            A tag table
+     */
 	public TextBuffer(TextTagTable table) {
 		super(gtk_text_buffer_new(table.getHandle()));
 	}
@@ -81,10 +95,7 @@ public class TextBuffer extends GObject {
 	 */
 	public TextTagTable getTextTagTable() {
 	    Handle hndl = gtk_text_buffer_get_tag_table(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (TextTagTable)obj;
-		return new TextTagTable(hndl);
+		return TextTagTable.getTextTagTable(hndl);
 	}
 
 	/**
@@ -337,7 +348,8 @@ public class TextBuffer extends GObject {
 	 * @return The new TextMark object
 	 */
 	public TextMark createMark(String markName, TextIter where, boolean leftGravity) {
-		return new TextMark(gtk_text_buffer_create_mark(getHandle(), markName, where.getHandle(), leftGravity));
+		return new TextMark(gtk_text_buffer_create_mark(getHandle(), markName,
+                where.getHandle(), leftGravity));
 	}
 
 	/**
@@ -393,12 +405,7 @@ public class TextBuffer extends GObject {
 	 */
 	public TextMark getMark(String name) {
 	    Handle markHandle = gtk_text_buffer_get_mark(getHandle(), name);
-		if (null == markHandle)
-			return null;
-		GObject obj = getGObjectFromHandle(markHandle);
-		if (null != obj)
-			return (TextMark)obj;
-		return new TextMark(markHandle);
+		return TextMark.getTextMark(markHandle);
 	}
 
 	/**
@@ -409,10 +416,7 @@ public class TextBuffer extends GObject {
 	 */
 	public TextMark getInsert() {
 	    Handle hndl = gtk_text_buffer_get_insert(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (TextMark)obj;
-		return new TextMark(hndl);
+		return TextMark.getTextMark(hndl);
 	}
 
 	/**
@@ -427,10 +431,7 @@ public class TextBuffer extends GObject {
 	 */
 	public TextMark getSelectionBound() {
 	    Handle hndl = gtk_text_buffer_get_selection_bound(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (TextMark)obj;
-		return new TextMark(hndl);
+		return TextMark.getTextMark(hndl);
 	}
 
 	/**
@@ -512,8 +513,9 @@ public class TextBuffer extends GObject {
 	 * @return Iterator
 	 */
 	public TextIter getIter(int lineNumber, int charOffset) {
-		return new TextIter(gtk_text_buffer_get_iter_at_line_offsetMOD(getHandle(), lineNumber, charOffset));
-	}
+		return TextIter.getTextIter(gtk_text_buffer_get_iter_at_line_offsetMOD(
+                getHandle(), lineNumber, charOffset));
+    }
 
 	/**
 	 * Obtains an iterator pointing to charOffset within the entire string,
@@ -522,7 +524,8 @@ public class TextBuffer extends GObject {
 	 * @return Iterator pointing to that point.
 	 */
 	public TextIter getIter(int charOffset) {
-		return new TextIter(gtk_text_buffer_get_iter_at_offsetMOD(getHandle(), charOffset));
+		return TextIter.getTextIter(gtk_text_buffer_get_iter_at_offsetMOD(
+                getHandle(), charOffset));
 	}
 
 	/**
@@ -531,7 +534,8 @@ public class TextBuffer extends GObject {
 	 * @return Iterator
 	 */
 	public TextIter getLineIter(int line) {
-		return new TextIter(gtk_text_buffer_get_iter_at_lineMOD(getHandle(), line));
+		return TextIter.getTextIter(gtk_text_buffer_get_iter_at_lineMOD(
+                getHandle(), line));
 	}
 
 	/**
@@ -540,7 +544,8 @@ public class TextBuffer extends GObject {
 	 * @return Iterator
 	 */
 	public TextIter getIter(TextMark mark) {
-		return new TextIter(gtk_text_buffer_get_iter_at_markMOD(getHandle(), mark.getHandle()));
+		return TextIter.getTextIter(gtk_text_buffer_get_iter_at_markMOD(
+                getHandle(), mark.getHandle()));
 	}
 
 	/**
@@ -549,7 +554,8 @@ public class TextBuffer extends GObject {
 	 * @return Iterator
 	 */
 	public TextIter getIter(TextChildAnchor anchor) {
-		return new TextIter(gtk_text_buffer_get_iter_at_child_anchorMOD(getHandle(), anchor.getHandle()));
+		return TextIter.getTextIter(gtk_text_buffer_get_iter_at_child_anchorMOD(
+                getHandle(), anchor.getHandle()));
 	}
 
 	/**
@@ -557,14 +563,14 @@ public class TextBuffer extends GObject {
 	 * @return iterator
 	 */
 	public TextIter getStartIter() {
-		return new TextIter(gtk_text_buffer_get_start_iterMOD(getHandle()));
+		return TextIter.getTextIter(gtk_text_buffer_get_start_iterMOD(getHandle()));
 	}
 	/**
 	 * Returns Iterator for the end of the text
 	 * @return Iterator for the end of the text.
 	 */
 	public TextIter getEndIter() {
-		return new TextIter(gtk_text_buffer_get_end_iterMOD(getHandle()));
+		return TextIter.getTextIter(gtk_text_buffer_get_end_iterMOD(getHandle()));
 	}
 
 	/**
@@ -657,8 +663,10 @@ public class TextBuffer extends GObject {
 	 */
 	public TextIter getSelectionStart() {
 		if (getSelected()) {
-			return new TextIter(gtk_text_buffer_get_selection_start(getHandle()));
-		} else {
+			return TextIter
+                    .getTextIter(gtk_text_buffer_get_selection_start(getHandle()));
+		}
+        else {
 			return null;
 		}
 	}
@@ -669,8 +677,9 @@ public class TextBuffer extends GObject {
 	 */
 	public TextIter getSelectionEnd() {
 		if (getSelected()) {
-			return new TextIter(gtk_text_buffer_get_selection_end(getHandle()));
-		} else {
+			return TextIter.getTextIter(gtk_text_buffer_get_selection_end(getHandle()));
+		}
+        else {
 			return null;
 		}
 	}
@@ -923,10 +932,14 @@ public class TextBuffer extends GObject {
 	}
 
 	/**
-	 * Creates a TextTag initialised for use in this TextBuffer.
+	 * Creates a TextTag initialised for use in this TextBuffer.  If
+	 * a TextTag already exists with this name null is returned.
 	 */
 	public TextTag createTag(String name) {
-		return new TextTag(create_tag(getHandle(), name));
+		Handle hndl = create_tag(getHandle(), name);
+		if (null == hndl)
+			return null;
+		return new TextTag(hndl);
 	}
 
 	native static final protected int gtk_text_buffer_get_type();
