@@ -8,8 +8,10 @@
  */
 
 package org.gnu.gtk;
-import org.gnu.glib.Boxed;
-public class TextLineData extends Boxed 
+
+import org.gnu.javagnome.Struct;
+
+public class TextLineData extends Struct 
 {
     /****************************************
      * BEGINNING OF GENERATED CODE
