@@ -11,13 +11,16 @@ package org.gnu.gtk;
 
 import java.util.Vector;
 
+import org.gnu.gdk.Point;
+import org.gnu.gdk.Rectangle;
+import org.gnu.gdk.Window;
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.TextViewEvent;
 import org.gnu.gtk.event.TextViewListener;
+import org.gnu.javagnome.Handle;
+import org.gnu.pango.TabArray;
 
 /**
  * Widget for displaying textual data.
@@ -89,9 +92,39 @@ import org.gnu.gtk.event.TextViewListener;
  */
 public class TextView extends Container {
 
+    /**
+     * Class to handle returns from
+     * 
+     * @see TextIter#getLineYRange TextIter.getLineYRange. Contains the starting
+     *      y-coordinates of a line and the height of the line
+     */
+    public final static class VerticalLineRange {
+        private final int y;
+
+        private final int height;
+
+        public VerticalLineRange(int y, int height) {
+            this.y = y;
+            this.height = height;
+        }
+
+        /**
+         * @return The y-coordinate of the line in buffer coordinates
+         */
+        public final int getY() {
+            return y;
+        }
+
+        /**
+         * @return The height of the line
+         */
+        public final int getHeight() {
+            return height;
+        }
+    }
 	/**
-	 * The list of objects interested in this widget's events
-	 */
+     * The list of objects interested in this widget's events
+     */
 	private Vector listeners;
 
 	/**
@@ -132,10 +165,7 @@ public class TextView extends Container {
 	 */
 	public TextBuffer getBuffer() {
 	    Handle hndl = gtk_text_view_get_buffer(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (TextBuffer)obj;
-		return new TextBuffer(hndl);
+		return TextBuffer.getTextBuffer(hndl);
 	}
 	/**
 	 * Scrolls the view so that mark is on the screen in the position indicated
@@ -147,8 +177,10 @@ public class TextView extends Container {
 	 * @param xAlign Horizontal alignment of mark within visible area.
 	 * @param yAlign : vertical alignment of mark within visible area
 	 */
-	public void scrollToMark(TextMark mark, double withinMargin, boolean useAlign, double xAlign, double yAlign) {
-		gtk_text_view_scroll_to_mark(getHandle(), mark.getHandle(), withinMargin, useAlign, xAlign, yAlign);
+	public void scrollToMark(TextMark mark, double withinMargin, boolean useAlign,
+            double xAlign, double yAlign) {
+		gtk_text_view_scroll_to_mark(getHandle(), mark.getHandle(), withinMargin,
+                useAlign, xAlign, yAlign);
 	}
 
 	/**
@@ -160,28 +192,88 @@ public class TextView extends Container {
 	 * @param withinMargin Margin as a fraction of screen size
 	 */
 	public void scrollToMark(TextMark mark, double withinMargin) {
-		gtk_text_view_scroll_to_mark(getHandle(), mark.getHandle(), withinMargin, false, 0.0, 0.0);
+		gtk_text_view_scroll_to_mark(getHandle(), mark.getHandle(), withinMargin,
+                false, 0.0, 0.0);
 	}
-
+	
+    /**
+     * Returns an iterator for the location pointed to by the coordinates 
+     * <code>x</code> and <code>y</code> within the buffer. Coordinates
+     * obtained in relation to the widget must be converted to buffer
+     * coordinates using windowToBufferCoords before passing them into
+     * this method.
+     * 
+     * @param x x buffer coordinate
+     * @param y y buffer coordinate
+     * @return an iterator to a given x-y coordinate location
+     */
+    public TextIter getIterAtLocation(int x, int y) {
+        Handle handle = gtk_text_view_get_iter_at_location(getHandle(), x, y);
+        return TextIter.getTextIter(handle);
+    }
+    
 	/**
-	 * Scrolls text view so that iter is on the screen in the position indicated
-	 * by xalign and yalign. An alignment of 0.0 indicates left or top, 1.0
-	 * indicates right or bottom, 0.5 means center. 
-	 * The effective screen for purposes of this function is
-	 * reduced by a margin of size within_margin. NOTE: This function uses the
-	 * currently-computed height of the lines in the text buffer. Note that line
-	 * heights are computed in an idle handler; so this function may not have
-	 * the desired effect if it's called before the height computations. To
-	 * avoid oddness, consider using gtk_text_view_scroll_to_mark() which saves
-	 * a point to be scrolled to after line validation.
-	 * @param iter a TextIter
-	 * @param withinMargin Margin as a [0.0,0.5) fraction of screen size.
-	 * @param xAlign : horizontal alignment of mark within visible area.
-	 * @param yAlign : vertical alignment of mark within visible area
-	 * @return TRUE if scrolling occurred
-	 */
-	public boolean scrollToIter(TextIter iter, double withinMargin, double xAlign, double yAlign) {
-		return gtk_text_view_scroll_to_iter(getHandle(), iter.getHandle(), withinMargin, true, xAlign, yAlign);
+     * Returns an iterator for the location pointed to by bufferCoords,
+     * representing coordinates within the buffer. Coordinates
+     * obtained in relation to the widget must be converted to buffer
+     * coordinates using windowToBufferCoords.
+     * 
+     * @param bufferCoords a Point representing a pair of buffer coordinates
+     * @return an iterator to a given x-y coordinate location
+     */
+    public TextIter getIterAtLocation(Point bufferCoords) {
+        Handle handle = gtk_text_view_get_iter_at_location(getHandle(), 
+                bufferCoords.getX(), bufferCoords.getY());
+        return TextIter.getTextIter(handle);
+    }
+    
+    /**
+     * Converts coordinates on the window identified by the <code>winType</code>
+     * to buffer coordinates.
+     * 
+     * @param winType a {@link TextWindowType}.
+     * @param xCoord the x-coordinate in relation to the Window win
+     * @param yCoord the y-coordinate in relation to the Window win
+     * @return a Point representing the buffer coordinates
+     */   
+    public Point windowToBufferCoords(TextWindowType winType, int xCoord,
+            int yCoord){
+        int[] bufferX = new int[1];
+        int [] bufferY = new int[1];
+        
+        gtk_text_view_window_to_buffer_coords(this.getHandle(),
+                winType.getValue(), xCoord , yCoord, bufferX, bufferY);
+        
+        return new Point(bufferX[0], bufferY[0]);
+    }
+    
+	/**
+     * Scrolls text view so that iter is on the screen in the position indicated
+     * by xalign and yalign. An alignment of 0.0 indicates left or top, 1.0
+     * indicates right or bottom, 0.5 means center. The effective screen for
+     * purposes of this function is reduced by a margin of size within_margin.
+     * NOTE: This function uses the currently-computed height of the lines in
+     * the text buffer. Note that line heights are computed in an idle handler;
+     * so this function may not have the desired effect if it's called before
+     * the height computations. To avoid oddness, consider using
+     * {@link #scrollToMark(TextMark, double, boolean, double, double) which
+     * saves a point to be scrolled to
+     * after line validation.
+     * 
+     * @param iter
+     *            a TextIter
+     * @param withinMargin
+     *            Margin as a [0.0,0.5) fraction of screen size.
+     * @param xAlign :
+     *            horizontal alignment of mark within visible area.
+     * @param yAlign :
+     *            vertical alignment of mark within visible area
+     * @return TRUE if scrolling occurred
+     */
+	public boolean scrollToIter(TextIter iter, double withinMargin,
+            double xAlign, double yAlign) {
+		return gtk_text_view_scroll_to_iter(getHandle(), iter.getHandle(),
+                withinMargin, true, xAlign, yAlign);
 	}
 
 	/**
@@ -198,9 +290,10 @@ public class TextView extends Container {
 	 * @return TRUE if scrolling occurred
 	 */
 	public boolean scrollToIter(TextIter iter, double withinMargin) {
-		return gtk_text_view_scroll_to_iter(getHandle(), iter.getHandle(), withinMargin, false, 0, 0);
+		return gtk_text_view_scroll_to_iter(getHandle(), iter.getHandle(),
+                withinMargin, false, 0, 0);
 	}
-
+    
 	/**
 	 * Moves the cursor to the currently visible region of the buffer, it it
 	 * isn't there already.
@@ -210,15 +303,33 @@ public class TextView extends Container {
 		return gtk_text_view_place_cursor_onscreen(getHandle());
 	}
 
+   /**
+    * Moves a mark within the buffer so that it's located within the 
+    * currently-visible text area.
+    * @return  TRUE if the mark had to be moved.
+    */
+    public boolean moveMarkOnScreen(TextMark mark) {
+        return gtk_text_view_move_mark_onscreen(getHandle(), mark.getHandle());
+    }
+    
 	/**
 	 * Adds a child widget in the text buffer, at the given anchor.
 	 * @param child A Widget
 	 * @param anchor A TextChildAnchor in the TextBuffer for his view
 	 */
 	public void addChild(Widget child, TextChildAnchor anchor) {
-		gtk_text_view_add_child_at_anchor(getHandle(), child.getHandle(), anchor.getHandle());
+		gtk_text_view_add_child_at_anchor(getHandle(), child.getHandle(),
+                anchor.getHandle());
 	}
 
+	/**
+	 * Gets the line wrapping for the view.
+	 * @return the line wrap setting for the view.
+	 */
+	public WrapMode getWrapMode() {
+		return WrapMode.intern(gtk_text_view_get_wrap_mode(getHandle()));
+	}
+	
 	/**
 	 * Sets the line wrapping for the view.
 	 * @param wrapMode A WrapMode
@@ -246,6 +357,14 @@ public class TextView extends Container {
 		return gtk_text_view_get_editable(getHandle());
 	}
 
+	/**
+	 * Find out whether the cursor is being displayed.
+	 * @return Whether the insertion cursor is visible
+	 */
+	public boolean getCursorVisible() {
+		return gtk_text_view_get_cursor_visible(getHandle());
+	}
+	
 	/**
 	 * Toggles whether the insertion point is displayed. A buffer with no
 	 * editable text probably shouldn't have a visible cursor, so you may want
@@ -256,6 +375,32 @@ public class TextView extends Container {
 		gtk_text_view_set_cursor_visible(getHandle(), setting);
 	}
 
+	/**
+	 * Gets the default number of pixels to put above paragraphs.
+	 * @return the default number of pixels to put above paragraphs.
+	 */
+	public int getPixelsAboveLines() {
+		return gtk_text_view_get_pixels_above_lines(getHandle());
+	}
+
+	/**
+	 * Gets the default number of pixels of blank space to put below paragraphs.
+	 * @return the default number of pixels of blank space to put below paragraphs
+	 */
+	public int getPixelsBelowLines() {
+		return gtk_text_view_get_pixels_below_lines(getHandle());
+	}
+
+	/**
+	 * Gets the default number of pixels of blank space to leave between 
+	 * display/wrapped lines within a paragraph. 
+	 * @return the default number of pixels of blank space to leave between 
+	 * display/wrapped lines within a paragraph. 
+	 */
+	public int getPixelsInsideWrap() {
+		return gtk_text_view_get_pixels_inside_wrap(getHandle());
+	}
+	
 	/**
 	 * Sets the default number of blank pixels above paragraphs in text_view.
 	 * Tags in the buffer for text_view may override the defaults.
@@ -269,11 +414,21 @@ public class TextView extends Container {
 	 * Sets the default number of pixels of blank space to put below paragraphs
 	 * in text_view. May be overridden by tags applied to text_view's buffer.
 	 * @param pixelsBelowLines Pixels below paragraphs
+     * @deprecated Use {@link #setPixelsBelowLines(int)} instead.
 	 */
 	public void setPixelsBelowLine(int pixelsBelowLines) {
 		gtk_text_view_set_pixels_below_lines(getHandle(), pixelsBelowLines);
 	}
 
+    /**
+     * Sets the default number of pixels of blank space to put below paragraphs
+     * in text_view. May be overridden by tags applied to text_view's buffer.
+     * @param pixelsBelowLines Pixels below paragraphs
+     */
+    public void setPixelsBelowLines(int pixelBelowLines) {
+        gtk_text_view_set_pixels_below_lines(getHandle(), pixelBelowLines);
+    }
+    
 	/**
 	 * Sets the default number of pixels of blank space to leave between
 	 * display/wrapped lines within a paragraph. May be overridden by tags in
@@ -284,6 +439,14 @@ public class TextView extends Container {
 		gtk_text_view_set_pixels_inside_wrap(getHandle(), pixelsInsideWrap);
 	}
 
+	/**
+	 * Gets the default justification of text in this TextView. Tags in the buffer may override the default.
+	 * @return The default justification
+	 */
+	public Justification getJustification(){
+		return Justification.intern(gtk_text_view_get_justification(this.getHandle()));
+	}
+	
 	/**
 	 * Sets the default justification of text in text_view. Tags in the view's
 	 * buffer may override the default.  
@@ -293,6 +456,24 @@ public class TextView extends Container {
 		gtk_text_view_set_justification(getHandle(), justification.getValue());
 	}
 
+	/**
+	 * Gets the default left margin size of paragraphs in the TextView. Tags
+	 * in the buffer may override the default.
+	 * @return  The left margin, in pixels
+	 */
+	public int getLeftMargin(){
+		return gtk_text_view_get_left_margin(getHandle());
+	}
+	
+	/**
+	 * Gets the default right margin size of paragraphs in the TextView. Tags
+	 * in the buffer may override the default.
+	 * @return The right margin, in pixels
+	 */
+	public int getRightMargin(){
+		return gtk_text_view_get_right_margin(getHandle());
+	}
+	
 	/**
 	 * Sets the default left margin for text in text_view. Tags in the buffer
 	 * may override the default.
@@ -311,6 +492,15 @@ public class TextView extends Container {
 		gtk_text_view_set_right_margin(getHandle(), rightMargin);
 	}
 
+	/**
+	 * Gets the default indentation of paragraphs in text_view. Tags in the 
+	 * view's buffer may override the default. The indentation may be negative.
+	 * @return The 	 number of pixels of indentation
+	 */
+	public int getIndent() {
+		return gtk_text_view_get_indent(getHandle());
+	}
+	
 	/**
 	 * Sets the default indentation for paragraphs in text_view. Tags in the
 	 * buffer may override the default.
@@ -320,6 +510,15 @@ public class TextView extends Container {
 		gtk_text_view_set_indent(getHandle(), indent);
 	}
 
+	/**
+	 * Gets the default tabs for text_view. Tags in the buffer may override 
+	 * the defaults. The returned array will be NULL if "standard" (8-space)
+	 *  tabs are used.
+	 */
+	public TabArray getTabs() {
+		return TabArray.getTabArray(gtk_text_view_get_tabs(getHandle()));
+	}
+	
 	/**
 	 * Sets the default tab stops for paragraphs in text_view. Tags in the
 	 * buffer may override the default.
@@ -336,7 +535,8 @@ public class TextView extends Container {
 	 * @return The text attributes being used
 	 */
 	public TextAttributes getDefaultAttributes() {
-		return new TextAttributes(gtk_text_view_get_default_attributes(getHandle()));
+        Handle handle = gtk_text_view_get_default_attributes(getHandle());
+        return TextAttributes.getTextAttributes(handle);
 	}
 
 	/**
@@ -384,11 +584,11 @@ public class TextView extends Container {
 	}
     
     public Adjustment getHAdjustment() {
-        return new Adjustment(getHAdjustment(getHandle()));
+        return Adjustment.getAdjustment(getHAdjustment(getHandle()));
     }
     
     public Adjustment getVAdjustment() {
-        return new Adjustment(getVAdjustment(getHandle())); 
+        return Adjustment.getAdjustment(getVAdjustment(getHandle())); 
     }
 
 	public Class getEventListenerClass(String signal) {
@@ -398,10 +598,132 @@ public class TextView extends Container {
 	}
 
 	public EventType getEventType(String signal) {
-		EventType et = evtMap.getEventType(signal);
-		if (et == null) et = super.getEventType(signal);
-		return et;
-	}
+        EventType et = evtMap.getEventType(signal);
+        if (et == null)
+            et = super.getEventType(signal);
+        return et;
+    }
+
+    /**
+     * Gets the y coordinate of the top of the line containing <code>iter</code>,
+     * and the height of the line. The coordinate is a buffer coordinate;
+     * convert to window coordinates with
+     * {@link #windowToBufferCoords(TextWindowType, int, int)}.
+     * 
+     * @param iter
+     *            A valid TextIter
+     * @return The y-coordinate and Height of the line containing iter
+     */
+    public VerticalLineRange getLineYRange(TextIter iter) {
+        int[] y = new int[1];
+        int[] height = new int[1];
+
+        gtk_text_view_get_line_yrange(getHandle(), iter.getHandle(), y, height);
+
+        return new VerticalLineRange(y[0], height[0]);
+    }
+
+    /**
+     * Converts coordinate (xCoord, yCoord) to coordinates for the window
+     * <code>winType</code>. Note that you can't convert coordinates for a
+     * non-existing window (see
+     * {@link TextView#setBorderWindowSize(TextWindowType, int)}.
+     *      
+     * @param winType a {@link TextWindowType}
+     * @param xCoord buffer X coordinate
+     * @param yCoord buffer Y coordinate
+     * @return the corresponding window coordinates
+     */
+    public Point bufferToWindowCoords(TextWindowType winType, int xCoord,
+            int yCoord) {
+        int[] windowX = new int[1];
+        int[] windowY = new int[1];
+        
+        gtk_text_view_buffer_to_window_coords(getHandle(), winType.getValue(),
+                xCoord, yCoord, windowX, windowY);
+
+        return new Point(windowX[0], windowY[0]);
+    }
+
+    /**
+     * Sets the width of the {@link TextWindowType#LEFT} or
+     * {@link TextWindowType#RIGHT} windows, or the height of
+     * {@link TextWindowType#TOP} or {@link TextWindowType#BOTTOM} Windows.
+     * Automatically destroys the corresponding window if the size is set to 0,
+     * and creates the window if the size is set to non-zero. This function can
+     * only be used for the "border windows," it doesn't work with
+     * {@link TextWindowType#WIDGET}, {@link TextWindowType#TEXT}, or
+     * {@link TextWindowType#PRIVATE}.
+     * 
+     * @param type
+     *            The window to affect
+     * @param size
+     *            Width or height of the window
+     */
+    public void setBorderWindowSize(TextWindowType type, int size) {
+        gtk_text_view_set_border_window_size(getHandle(), type.getValue(), size);
+    }
+
+    /**
+     * Gets the width of the specified border window.
+     * 
+     * @param type
+     *            The window to return size from
+     * @return The size of the window
+     * 
+     * @see TextView#setBorderWindowSize(TextWindowType, int).
+     */
+    public int getBorderWindowSize(TextWindowType type) {
+        return gtk_text_view_get_border_window_size(getHandle(), type
+                .getValue());
+    }
+
+    /**
+     * Retrieves the {@link org.gnu.gdk.Window} corresponding to an area of the
+     * text view; possible windows include the overall widget window, child
+     * windows on the left, right, top, bottom, and the window that displays the
+     * text buffer. Windows are null and nonexistent if their width or height is
+     * 0, and are nonexistent before the widget has been realized.
+     * 
+     * @param type
+     *            The window to get
+     * @return The window, or null
+     */
+    public Window getWindow(TextWindowType type) {
+        Handle hndl = gtk_text_view_get_window(getHandle(), type.getValue());
+        return Window.getWindowFromHandle(hndl);
+    }
+
+    /**
+     * Usually used to find out which window an event corresponds to.
+     * 
+     * @param win
+     *            A window
+     * @return The type of the window
+     */
+    public TextWindowType getWindowType(Window win) {
+        return TextWindowType.intern(gtk_text_view_get_window_type(getHandle(),
+                win.getHandle()));
+    }
+    
+    /**
+     * Scrolls this the minimum distance such that mark is contained within
+     * the visible area of the widget.
+     * 
+     * @param mark a mark in the buffer of the TextView.
+     */ 
+    public void scrollMarkOnScreen(TextMark mark) {
+        gtk_text_view_scroll_mark_onscreen(this.getHandle(), mark.getHandle());
+    }
+    
+    /**
+     * Returns a Rectangle with the currently-visible region of the buffer, in
+     * buffer coordinates. Convert to window coordinates with 
+     * {@link #bufferToWindowCoords(TextWindowType, int, int)}.
+     */
+    public Rectangle getVisibleRectangle() {
+        return Rectangle.getRectangle(gtk_text_view_get_visible_rect(this.getHandle()));
+    }
 
 	private static EventMap evtMap = new EventMap();
 	static {
@@ -409,9 +731,9 @@ public class TextView extends Container {
 	}
 
 	/**
-	* Implementation method to build an EventMap for this widget class.
-	* Not useful (or supported) for application use.
-	*/
+     * Implementation method to build an EventMap for this widget class. Not
+     * useful (or supported) for application use.
+     */
 	private static void addEvents(EventMap anEvtMap) {
 		anEvtMap.addEvent("copy_clipboard", "handleCopyClipboard", TextViewEvent.Type.COPY_CLIPBOARD, TextViewListener.class);
 		anEvtMap.addEvent("cut_clipboard", "handleCutClipboard", TextViewEvent.Type.CUT_CLIPBOARD, TextViewListener.class);
@@ -537,11 +859,7 @@ public class TextView extends Container {
 
 	private void handlePopulatePopup(Handle menu) {
 		TextViewEvent tve = new TextViewEvent(this, TextViewEvent.Type.POPULATE_POPUP);
-		GObject obj = getGObjectFromHandle(menu);
-		if (null != obj)
-			tve.setMenu((Menu)obj);
-		else
-			tve.setMenu(new Menu(menu));
+		tve.setMenu(Menu.getMenu(menu));
 		fireTextViewEvent(tve);
 	}
 
@@ -576,11 +894,11 @@ public class TextView extends Container {
 	native static final protected void gtk_text_view_scroll_mark_onscreen(Handle textView, Handle mark);
 	native static final protected boolean gtk_text_view_move_mark_onscreen(Handle textView, Handle mark);
 	native static final protected boolean gtk_text_view_place_cursor_onscreen(Handle textView);
-	native static final protected void gtk_text_view_get_visible_rect(Handle textView, Handle visibleRect);
+	native static final protected Handle gtk_text_view_get_visible_rect(Handle textView);
 	native static final protected void gtk_text_view_set_cursor_visible(Handle textView, boolean setting);
 	native static final protected boolean gtk_text_view_get_cursor_visible(Handle textView);
-	native static final protected void gtk_text_view_get_iter_location(Handle textView, Handle iter, int location);
-	native static final protected void gtk_text_view_get_iter_at_location(Handle textView, Handle iter, int x, int y);
+	native static final protected Handle gtk_text_view_get_iter_location(Handle textView, Handle iter);
+	native static final protected Handle gtk_text_view_get_iter_at_location(Handle textView, int x, int y);
 	native static final protected void gtk_text_view_get_line_yrange(Handle textView, Handle iter, int[] y, int[] height);
 	native static final protected void gtk_text_view_get_line_at_y(Handle textView, Handle targetIter, int y, int[] lneTop);
 	native static final protected void gtk_text_view_buffer_to_window_coords(Handle textView,int win,int bufferX,int bufferY,int[] windowX,int[] windowY);
