@@ -8,7 +8,7 @@
  */
 package org.gnu.gtk;
 
-import org.gnu.glib.Handle;
+import org.gnu.javagnome.Handle;
 
 /**
  */
