@@ -13,10 +13,10 @@ import java.util.Vector;
 
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.ToggleEvent;
 import org.gnu.gtk.event.ToggleListener;
+import org.gnu.javagnome.Handle;
 
 /**
  * A ToggleButton is a button that changes and retains its on or off
