@@ -13,11 +13,10 @@ import java.util.Vector;
 
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
-import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.TreeViewEvent;
 import org.gnu.gtk.event.TreeViewListener;
+import org.gnu.javagnome.Handle;
 
 /**
  * The TreeView object is a widget for displaying trees and lists. The sole
@@ -124,6 +123,20 @@ public class TreeView extends Container {
 	public TreeView(Handle handle) {
 		super(handle);
 	}
+    
+    /**
+     * Creates a new tree view from a native handle
+     */
+    public static TreeView getTreeView(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        TreeView obj = (TreeView) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new TreeView(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Creates a new TreeView Widget with the initial model set
@@ -137,10 +150,7 @@ public class TreeView extends Container {
 	 */
 	public TreeModel getModel() {
 	    Handle hndl = gtk_tree_view_get_model(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (TreeModel)obj;
-		return new TreeModel(hndl);
+		return TreeModel.getTreeModel(hndl);
 	}
 
 	/**
@@ -178,10 +188,7 @@ public class TreeView extends Container {
 	 */
 	public TreeSelection getSelection() {
 	    Handle hndl = gtk_tree_view_get_selection(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (TreeSelection)obj;
-		return new TreeSelection(hndl);
+		return TreeSelection.getTreeSelection(hndl);
 	}
 
 	/**
@@ -190,10 +197,7 @@ public class TreeView extends Container {
 	 */
 	public Adjustment getHAdjustment() {
 	    Handle hndl = gtk_tree_view_get_hadjustment(getHandle());
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Adjustment)obj;
-		return new Adjustment(hndl);
+		return Adjustment.getAdjustment(hndl);
 	}
 
 	/**
@@ -207,7 +211,7 @@ public class TreeView extends Container {
 	 * Gets the Adjustment currently being used for the vertical aspect.
 	 */
 	public Adjustment getVAdjustment() {
-		return new Adjustment(gtk_tree_view_get_vadjustment(getHandle()));
+		return Adjustment.getAdjustment(gtk_tree_view_get_vadjustment(getHandle()));
 	}
 
 	/**
@@ -270,12 +274,7 @@ public class TreeView extends Container {
 	 */
 	public TreeViewColumn getColumn(int position) {
 	    Handle h = gtk_tree_view_get_column(getHandle(), position);
-		if (h == null)
-			return null;
-		GObject obj = getGObjectFromHandle(h);
-		if (null != obj)
-			return (TreeViewColumn)obj;
-		return new TreeViewColumn(h);
+		return TreeViewColumn.getTreeViewColumn(h);
 	}
 
 	/**
@@ -301,12 +300,7 @@ public class TreeView extends Container {
 	
 	public TreeViewColumn getExpanderColumn() {
 	    Handle hndl = gtk_tree_view_get_expander_column(getHandle());
-		if (hndl == null)
-			return null;
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (TreeViewColumn)obj;
-		return new TreeViewColumn(hndl);
+		return TreeViewColumn.getTreeViewColumn(hndl);
 	}
 
 	/**
@@ -391,21 +385,18 @@ public class TreeView extends Container {
 	 * Returns the current path
 	 */
 	public TreePath getCursorPath() {
-		Handle path = GObject.getNullHandle();
+		Handle path = getNullHandle();
 		gtk_tree_view_get_cursor(getHandle(), path, null);
-		return new TreePath(path);
+		return TreePath.getTreePath(path);
 	}
 
 	/** 
 	 * Returns the current column 
 	 */
 	public TreeViewColumn getCursorColumn() {
-		Handle column = GObject.getNullHandle();
+		Handle column = getNullHandle();
 		gtk_tree_view_get_cursor(getHandle(), null, column);
-		GObject obj = getGObjectFromHandle(column);
-		if (null != obj)
-			return (TreeViewColumn)obj;
-		return new TreeViewColumn(column);
+		return TreeViewColumn.getTreeViewColumn(column);
 	}
 
 	/**
@@ -514,15 +505,604 @@ public class TreeView extends Container {
 			return null;
 		TreeViewColumn[] columns = new TreeViewColumn[hndls.length];
 		for (int i = 0; i < hndls.length; i++) {
-			GObject obj = getGObjectFromHandle(hndls[i]);
-			if (null != obj)
-				columns[i] = (TreeViewColumn)obj;
-			else
-				columns[i] = new TreeViewColumn(hndls[i]);
+			columns[i] = TreeViewColumn.getTreeViewColumn(hndls[i]);
 		}
 		return columns;
 	}
 
+    
+    /**
+     * <p>Shows a data column of the model in this <code>TreeView</code>. This
+     * method will try to guess the renderer to use, and because of that,
+     * it has some limitations. For instance, you should only use it to
+     * show these kinds of DataColumns: <code>DataColumnBoolean</code>,
+     * <code>DataColumnString</code>, <code>DataColumnPixbuf</code>,
+     * <code>DataColumnStockItem</code>.</p>
+     * <p>
+     * For other kinds of DataColumn, the method 
+     * <code>showDataColumn(int, CellRenderer, CellRendererAttribute)</code>
+     * should be used, which is more flexible, allowing you to specify which 
+     * renderer to use, and its rendering attributes.
+     * </p> 
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @see #showDataColumn(int, boolean)
+     * @see #showDataColumn(int, CellRenderer, CellRendererAttribute)
+     * @see DataColumn
+     * @return an instance to the new shown column.
+     */
+	public TreeViewColumn showDataColumn(int aCol) {
+
+        TreeViewColumn viewCol = null;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = null;
+            CellRendererAttribute rendererAttr  = null;
+            
+            if (dataCol instanceof DataColumnBoolean) {
+                renderer = new CellRendererToggle();
+                rendererAttr = CellRendererToggle.Attribute.RADIO;
+            
+            } else if (dataCol instanceof DataColumnString) {
+                renderer = new CellRendererText();
+                rendererAttr = CellRendererText.Attribute.TEXT;
+            
+            } else if (dataCol instanceof DataColumnPixbuf) {
+                renderer = new CellRendererPixbuf();
+                rendererAttr = CellRendererPixbuf.Attribute.PIXBUF;
+            
+            } else if (dataCol instanceof DataColumnStockItem) {
+                renderer = new CellRendererPixbuf();
+                rendererAttr = CellRendererPixbuf.Attribute.STOCK_ID;
+            
+            } else {
+                throw new IllegalArgumentException("Since column " + aCol + "isn't a " +
+                        "DataColumnBoolean, DataColumnString, DataColumnPixbuf," +
+                        "or DataColumnStockItem, " +
+                        "Java-Gnome can't guess which renderer to use; " +
+                        "thesefore you should explicitly specify the renderer");
+            }
+            
+            viewCol  = new TreeViewColumn();
+            viewCol.packStart(renderer, false);
+            viewCol.addAttributeMapping(renderer, rendererAttr, dataCol);
+            
+            this.appendColumn(viewCol);
+        }
+        
+        return viewCol;
+    }
+    
+    
+	/**
+     * <p>Shows a data column of the model in this <code>TreeView</code>. This
+     * method will try to guess the renderer to use, and because of that,
+     * it has some limitations. For instance, you should only use it to
+     * show these kinds of DataColumns: <code>DataColumnBoolean</code>,
+     * <code>DataColumnString</code>, <code>DataColumnPixbuf</code>,
+     * <code>DataColumnStockItem</code>.</p>
+     * <p>
+     * For other kinds of DataColumn, the method 
+     * <code>showDataColumn(int, CellRenderer, CellRendererAttribute)</code>
+     * should be used, which is more flexible, allowing you to specify which 
+     * renderer to use, and its rendering attributes.
+     * </p> 
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param expand <code>true</code> if this column should expand to fill
+     * the available space in the row, <code>false</code> otherwise.
+     * @see #showDataColumn(int, boolean)
+     * @see #showDataColumn(int, CellRenderer, CellRendererAttribute)
+     * @see DataColumn
+     * @return an instance to the new shown column.
+     */
+	public TreeViewColumn showDataColumn(int aCol, boolean expand) {
+
+        TreeViewColumn viewCol = null;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = null;
+            CellRendererAttribute rendererAttr  = null;
+            
+            if (dataCol instanceof DataColumnBoolean) {
+                renderer = new CellRendererToggle();
+                rendererAttr = CellRendererToggle.Attribute.RADIO;
+            
+            } else if (dataCol instanceof DataColumnString) {
+                renderer = new CellRendererText();
+                rendererAttr = CellRendererText.Attribute.TEXT;
+            
+            } else if (dataCol instanceof DataColumnPixbuf) {
+                renderer = new CellRendererPixbuf();
+                rendererAttr = CellRendererPixbuf.Attribute.PIXBUF;
+            
+            } else if (dataCol instanceof DataColumnStockItem) {
+                renderer = new CellRendererPixbuf();
+                rendererAttr = CellRendererPixbuf.Attribute.STOCK_ID;
+            
+            } else {
+                throw new IllegalArgumentException("Since column " + aCol + "isn't a " +
+                        "DataColumnBoolean, DataColumnString, DataColumnPixbuf," +
+                        "or DataColumnStockItem, " +
+                        "Java-Gnome can't guess which renderer to use; " +
+                        "thesefore you should explicitly specify the renderer");
+            }
+            
+            viewCol  = new TreeViewColumn();
+            viewCol.packStart(renderer, expand);
+            viewCol.addAttributeMapping(renderer, rendererAttr, dataCol);
+            
+            this.appendColumn(viewCol);
+        }
+        
+        return viewCol;
+    }
+    
+    
+	/**
+     * Shows a data column of the model in this <code>TreeView</code>, with
+     * a specific renderer and rendering attributes.  
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param aRenderer the renderer to be used
+     * @param aAttr the rendering attributes to be used
+     * @see #showDataColumn(int)
+     * @see #showDataColumn(int, boolean)
+     * @see DataColumn
+     * @return an instance to the new shown column.
+     */
+	public TreeViewColumn showDataColumn(int aCol, CellRenderer aRenderer, CellRendererAttribute aAttr) {
+
+        TreeViewColumn viewCol = null;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = aRenderer;
+            CellRendererAttribute rendererAttr  = aAttr;
+            
+            viewCol  = new TreeViewColumn();
+            viewCol.packStart(renderer, false);
+            viewCol.addAttributeMapping(renderer, rendererAttr, dataCol);
+            
+            this.appendColumn(viewCol);
+        }
+        
+        return viewCol;
+    }
+    
+	/**
+     * Shows a data column of the model in this <code>TreeView</code>, with
+     * a specific renderer and rendering attributes.  
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param aRenderer the renderer to be used
+     * @param aAttr the rendering attributes to be used
+     * @param expand <code>true</code> if this column should expand to fill
+     * the available space in the row, <code>false</code> otherwise.
+     * @see #showDataColumn(int)
+     * @see #showDataColumn(int, boolean)
+     * @see DataColumn
+     * @return an instance to the new shown column.
+     */
+    public TreeViewColumn showDataColumn(int aCol, CellRenderer aRenderer, 
+                                         CellRendererAttribute aAttr, 
+                                         boolean expand) {
+
+        TreeViewColumn viewCol = null;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = aRenderer;
+            CellRendererAttribute rendererAttr  = aAttr;
+            
+            viewCol  = new TreeViewColumn();
+            viewCol.packStart(renderer, expand);
+            viewCol.addAttributeMapping(renderer, rendererAttr, dataCol);
+            
+            this.appendColumn(viewCol);
+        }
+        
+        return viewCol;
+    }
+    
+    /**
+     * Shows a data column of the model in this <code>TreeView</code>, with
+     * a specific renderer and rendering attributes.  
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param aRenderer the renderer to be used
+     * @param aAttr the rendering attributes to be used
+     * @see #showDataColumn(int)
+     * @see #showDataColumn(int, boolean)
+     * @see DataColumn
+     * @return an instance to the new shown column.
+     */
+    public TreeViewColumn showDataColumn(int aCol, CellRenderer aRenderer, 
+                                         CellRendererAttribute aAttr[]) {
+
+        TreeViewColumn viewCol = null;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer   = aRenderer;
+            
+            viewCol  = new TreeViewColumn();
+            viewCol.packStart(renderer, false);
+            
+            for (int i=0; aCol < aAttr.length; aCol++) {
+                viewCol.addAttributeMapping(renderer, aAttr[i], dataCol);
+            }
+            
+            this.appendColumn(viewCol);
+        }
+        
+        return viewCol;
+    }
+    
+    /**
+     * Shows a data column of the model in this <code>TreeView</code>, with
+     * a specific renderer and rendering attributes.  
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param aRenderer the renderer to be used
+     * @param aAttr the rendering attributes to be used
+     * @param expand <code>true</code> if this column should expand to fill
+     * the available space in the row, <code>false</code> otherwise.
+     * @see #showDataColumn(int)
+     * @see #showDataColumn(int, boolean)
+     * @see DataColumn
+     * @return an instance to the new shown column.
+     */
+    public TreeViewColumn showDataColumn(int aCol, CellRenderer aRenderer, 
+                                         CellRendererAttribute aAttr[],
+                                         boolean expand) {
+
+        TreeViewColumn viewCol = null;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer   = aRenderer;
+            
+            viewCol  = new TreeViewColumn();
+            viewCol.packStart(renderer, expand);
+            
+            for (int i=0; aCol < aAttr.length; aCol++) {
+                viewCol.addAttributeMapping(renderer, aAttr[i], dataCol);
+            }
+            
+            this.appendColumn(viewCol);
+        }
+        
+        return viewCol;
+    }
+    
+	/**
+     * <p>Appends a data column of the model to an existing 
+     * <code>TreeViewColumn</code>, previously created with one of the 
+     * <code>showDataColumn</code> methods. It should be used when it's desired 
+     * to have more than one data in just one column.</p>
+     * <p>
+     * This method will try to guess the 
+     * renderer to use, and because of that, it has some limitations. 
+     * For instance, you should only use it to show these kinds of DataColumns: 
+     * <code>DataColumnBoolean</code>, <code>DataColumnString</code>, 
+     * <code>DataColumnPixbuf</code>,
+     * <code>DataColumnStockItem</code>.</p>
+     * <p>
+     * For other kinds of DataColumn, the method 
+     * <code>appendDataColumn(TreeViewColumn, int, CellRenderer, CellRendererAttribute)</code>
+     * should be used, which is more flexible, allowing you to specify which 
+     * renderer to use, and its rendering attributes.
+     * </p> 
+     * @param aExistingCol an existing <code>TreeViewColumn</code>.
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @see #showDataColumn(int)
+     * @see #appendDataColumn(TreeViewColumn, int, CellRenderer, CellRendererAttribute)
+     * @see DataColumn
+     * @return an instance to the updated column.
+     */
+	public TreeViewColumn appendDataColumn(TreeViewColumn aExistingCol, int aCol) {
+
+        TreeViewColumn viewCol = aExistingCol;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = null;
+            CellRendererAttribute rendererAttr  = null;
+            
+            if (dataCol instanceof DataColumnBoolean) {
+                renderer = new CellRendererToggle();
+                rendererAttr = CellRendererToggle.Attribute.RADIO;
+            
+            } else if (dataCol instanceof DataColumnString) {
+                renderer = new CellRendererText();
+                rendererAttr = CellRendererText.Attribute.TEXT;
+            
+            } else if (dataCol instanceof DataColumnPixbuf) {
+                renderer = new CellRendererPixbuf();
+                rendererAttr = CellRendererPixbuf.Attribute.PIXBUF;
+            
+            } else if (dataCol instanceof DataColumnStockItem) {
+                renderer = new CellRendererPixbuf();
+                rendererAttr = CellRendererPixbuf.Attribute.STOCK_ID;
+            
+            } else {
+                throw new IllegalArgumentException("Since column " + aCol + "isn't a " +
+                        "DataColumnBoolean, DataColumnString, DataColumnPixbuf," +
+                        "or DataColumnStockItem, " +
+                        "Java-Gnome can't guess which renderer to use; " +
+                        "thesefore you should explicitly specify the renderer");
+            }
+            
+            viewCol.packStart(renderer, false);
+            viewCol.addAttributeMapping(renderer, rendererAttr, dataCol);
+        }
+        
+        return viewCol;
+    }
+    
+/**
+     * <p>Appends a data column of the model to an existing 
+     * <code>TreeViewColumn</code>, previously created with one of the 
+     * <code>showDataColumn</code> methods. It should be used when it's desired 
+     * to have more than one data in just one column.</p>
+     * <p>
+     * This method will try to guess the 
+     * renderer to use, and because of that, it has some limitations. 
+     * For instance, you should only use it to show these kinds of DataColumns: 
+     * <code>DataColumnBoolean</code>, <code>DataColumnString</code>, 
+     * <code>DataColumnPixbuf</code>,
+     * <code>DataColumnStockItem</code>.</p>
+     * <p>
+     * For other kinds of DataColumn, the method 
+     * <code>appendDataColumn(TreeViewColumn, int, CellRenderer, CellRendererAttribute)</code>
+     * should be used, which is more flexible, allowing you to specify which 
+     * renderer to use, and its rendering attributes.
+     * </p> 
+     * @param aExistingCol an existing <code>TreeViewColumn</code>.
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param expand <code>true</code> if this column should expand to fill
+     * the available space in the row, <code>false</code> otherwise.
+     * @see #showDataColumn(int)
+     * @see #appendDataColumn(TreeViewColumn, int, CellRenderer, CellRendererAttribute)
+     * @see DataColumn
+     * @return an instance to the updated column.
+     */
+    public TreeViewColumn appendDataColumn(TreeViewColumn aExistingCol, int aCol,
+                                           boolean expand) {
+
+        TreeViewColumn viewCol = aExistingCol;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = null;
+            CellRendererAttribute rendererAttr  = null;
+            
+            if (dataCol instanceof DataColumnBoolean) {
+                renderer = new CellRendererToggle();
+                rendererAttr = CellRendererToggle.Attribute.RADIO;
+            
+            } else if (dataCol instanceof DataColumnString) {
+                renderer = new CellRendererText();
+                rendererAttr = CellRendererText.Attribute.TEXT;
+            
+            } else if (dataCol instanceof DataColumnPixbuf) {
+                renderer = new CellRendererPixbuf();
+                rendererAttr = CellRendererPixbuf.Attribute.PIXBUF;
+            
+            } else if (dataCol instanceof DataColumnStockItem) {
+                renderer = new CellRendererPixbuf();
+                rendererAttr = CellRendererPixbuf.Attribute.STOCK_ID;
+            
+            } else {
+                throw new IllegalArgumentException("Since column " + aCol + "isn't a " +
+                        "DataColumnBoolean, DataColumnString, DataColumnPixbuf," +
+                        "or DataColumnStockItem, " +
+                        "Java-Gnome can't guess which renderer to use; " +
+                        "thesefore you should explicitly specify the renderer");
+            }
+            
+            viewCol.packStart(renderer, expand);
+            viewCol.addAttributeMapping(renderer, rendererAttr, dataCol);
+        }
+        
+        return viewCol;
+    }
+    
+	/**
+     * <p>
+     * Appends a data column of the model to an existing 
+     * <code>TreeViewColumn</code>, previously created with one of the 
+     * <code>showDataColumn</code> methods, and lets you use
+     * a specific renderer and rendering attributes. </p>
+     * <p>
+     * It should be used when it's desired 
+     * to have more than one data in just one column.</p>  
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param aRenderer the renderer to be used
+     * @param aAttr the rendering attributes to be used
+     * @see #showDataColumn(int)
+     * @see #appendDataColumn(TreeViewColumn, int)
+     * @see DataColumn
+     * @return an instance to the updated column.
+     */
+	public TreeViewColumn appendDataColumn(TreeViewColumn aExistingCol, int aCol, 
+                                           CellRenderer aRenderer, 
+                                           CellRendererAttribute aAttr) {
+
+        TreeViewColumn viewCol = aExistingCol;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = aRenderer;
+            CellRendererAttribute rendererAttr  = aAttr;
+            
+            viewCol.packStart(renderer, false);
+            viewCol.addAttributeMapping(renderer, rendererAttr, dataCol);
+        }
+        
+        return viewCol;
+    }
+    
+	/**
+     * <p>
+     * Appends a data column of the model to an existing 
+     * <code>TreeViewColumn</code>, previously created with one of the 
+     * <code>showDataColumn</code> methods, and lets you use
+     * a specific renderer and rendering attributes. </p>
+     * <p>
+     * It should be used when it's desired 
+     * to have more than one data in just one column.</p>  
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param aRenderer the renderer to be used
+     * @param aAttr the rendering attributes to be used
+     * @param expand <code>true</code> if this column should expand to fill
+     * the available space in the row, <code>false</code> otherwise.
+     * @see #showDataColumn(int)
+     * @see #appendDataColumn(TreeViewColumn, int)
+     * @see DataColumn
+     * @return an instance to the updated column.
+     */
+    public TreeViewColumn appendDataColumn(TreeViewColumn aExistingCol, int aCol, 
+                                           CellRenderer aRenderer, 
+                                           CellRendererAttribute aAttr,
+                                           boolean expand) {
+
+        TreeViewColumn viewCol = aExistingCol;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = aRenderer;
+            CellRendererAttribute rendererAttr  = aAttr;
+            
+            viewCol.packStart(renderer, expand);
+            viewCol.addAttributeMapping(renderer, rendererAttr, dataCol);
+        }
+        
+        return viewCol;
+    }
+    
+    /**
+     * <p>
+     * Appends a data column of the model to an existing 
+     * <code>TreeViewColumn</code>, previously created with one of the 
+     * <code>showDataColumn</code> methods, and lets you use
+     * a specific renderer and rendering attributes. </p>
+     * <p>
+     * It should be used when it's desired 
+     * to have more than one data in just one column.</p>  
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param aRenderer the renderer to be used
+     * @param aAttr the rendering attributes to be used
+     * @see #showDataColumn(int)
+     * @see #appendDataColumn(TreeViewColumn, int)
+     * @see DataColumn
+     * @return an instance to the updated column.
+     */
+    public TreeViewColumn appendDataColumn(TreeViewColumn aExistingCol, int aCol, 
+                                           CellRenderer aRenderer, 
+                                           CellRendererAttribute aAttr[]) {
+
+        TreeViewColumn viewCol = aExistingCol;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = aRenderer;
+            
+            viewCol.packStart(renderer, false);
+            
+            for (int i=0; i<aAttr.length; i++) {
+                viewCol.addAttributeMapping(renderer, aAttr[i], dataCol);
+            }
+        }
+        
+        return viewCol;
+    }
+    
+    /**
+     * <p>
+     * Appends a data column of the model to an existing 
+     * <code>TreeViewColumn</code>, previously created with one of the 
+     * <code>showDataColumn</code> methods, and lets you use
+     * a specific renderer and rendering attributes. </p>
+     * <p>
+     * It should be used when it's desired 
+     * to have more than one data in just one column.</p>  
+     * @param aCol the number of the column in the model to be shown; starting
+     * from 0.
+     * @param aRenderer the renderer to be used
+     * @param aAttr the rendering attributes to be used
+     * @param expand <code>true</code> if this column should expand to fill
+     * the available space in the row, <code>false</code> otherwise.
+     * @see #showDataColumn(int)
+     * @see #appendDataColumn(TreeViewColumn, int)
+     * @see DataColumn
+     * @return an instance to the updated column.
+     */
+    public TreeViewColumn appendDataColumn(TreeViewColumn aExistingCol, int aCol, 
+                                           CellRenderer aRenderer, 
+                                           CellRendererAttribute aAttr[],
+                                           boolean expand) {
+
+        TreeViewColumn viewCol = aExistingCol;
+        TreeModel model = getModel();
+        
+        if (model instanceof ListStore) {
+            
+            ListStore store         = (ListStore) model;
+            DataColumn dataCol      = store.getDataColumn(aCol);
+            CellRenderer renderer               = aRenderer;
+            
+            viewCol.packStart(renderer, expand);
+            
+            for (int i=0; i<aAttr.length; i++) {
+                viewCol.addAttributeMapping(renderer, aAttr[i], dataCol);
+            }
+        }
+        
+        return viewCol;
+    }
+    
 	/**
 	 * Sets the current keyboard focus to be at path, and selects it. This is 
 	 * useful when you want to focus the user's attention on a particular 
@@ -697,9 +1277,9 @@ public class TreeView extends Container {
     }
 
     protected boolean handleSeparatorFunc( Handle model, Handle iter ) {
-        TreeModel mod = (TreeModel)getGObjectFromHandle(model);
-        TreeIter it = new TreeIter( iter, mod );
-        return separatorFunc.isSeparator( mod, it );
+        TreeModel mod = (TreeModel) getGObjectFromHandle(model);
+        TreeIter it = TreeIter.getTreeIter(iter, mod);
+        return separatorFunc.isSeparator(mod, it);
     }
 
     /**
@@ -734,7 +1314,7 @@ public class TreeView extends Container {
 				evtMap.initialize(this, TreeViewEvent.Type.COLUMNS_CHANGED);
 				evtMap.initialize(this, TreeViewEvent.Type.EXPAND_COLLAPSE_CURSOR_ROW);
 				evtMap.initialize(this, TreeViewEvent.Type.ROW_ACTIVATED);
-				evtMap.initialize(this, TreeViewEvent.Type.ROW_COLLAPSED);
+				evtMap.initialize(this, TreeViewEvent.Type.ROW_EXPANDED);
 				evtMap.initialize(this, TreeViewEvent.Type.ROW_COLLAPSED);
 				evtMap.initialize(this, TreeViewEvent.Type.SELECT_ALL);
 				evtMap.initialize(this, TreeViewEvent.Type.SELECT_CURSOR_PARENT);
@@ -761,7 +1341,7 @@ public class TreeView extends Container {
 			evtMap.uninitialize(this, TreeViewEvent.Type.COLUMNS_CHANGED);
 			evtMap.uninitialize(this, TreeViewEvent.Type.EXPAND_COLLAPSE_CURSOR_ROW);
 			evtMap.uninitialize(this, TreeViewEvent.Type.ROW_ACTIVATED);
-			evtMap.uninitialize(this, TreeViewEvent.Type.ROW_COLLAPSED);
+			evtMap.uninitialize(this, TreeViewEvent.Type.ROW_EXPANDED);
 			evtMap.uninitialize(this, TreeViewEvent.Type.ROW_COLLAPSED);
 			evtMap.uninitialize(this, TreeViewEvent.Type.SELECT_ALL);
 			evtMap.uninitialize(this, TreeViewEvent.Type.SELECT_CURSOR_PARENT);
@@ -801,28 +1381,25 @@ public class TreeView extends Container {
 
 	private void handleRowActivated(Handle path, Handle column) {
 		TreeViewEvent event = new TreeViewEvent(this, TreeViewEvent.Type.ROW_ACTIVATED);
-		TreePath aPath = new TreePath(path);
+		TreePath aPath = TreePath.getTreePath(path);
 		event.setTreePath(aPath);
 		event.setTreeIter(this.getModel().getIter(aPath));
-		GObject obj = getGObjectFromHandle(column);
-		if (null != obj)
-			event.setTreeColumn((TreeViewColumn)obj);
-		else
-			event.setTreeColumn(new TreeViewColumn(column));
+		event.setTreeColumn(TreeViewColumn.getTreeViewColumn(column));
 		fireTreeViewEvent(event);
 	}
 
 	private void handleRowExpanded(Handle iter, Handle path) {
 		TreeViewEvent event = new TreeViewEvent(this, TreeViewEvent.Type.ROW_EXPANDED);
-		event.setTreePath(new TreePath(path));
-		event.setTreeIter(new TreeIter(iter, getModel()));
+		event.setTreePath(TreePath.getTreePath(path));
+		event.setTreeIter(TreeIter.getTreeIter(iter, getModel()));
+		event.setExpand(true);
 		fireTreeViewEvent(event);
 	}
 
 	private void handleRowCollapsed(Handle iter, Handle path) {
 		TreeViewEvent event = new TreeViewEvent(this, TreeViewEvent.Type.ROW_COLLAPSED);
-		event.setTreePath(new TreePath(path));
-		event.setTreeIter(new TreeIter(iter, getModel()));
+		event.setTreePath(TreePath.getTreePath(path));
+		event.setTreeIter(TreeIter.getTreeIter(iter, getModel()));
 		fireTreeViewEvent(event);
 	}
 
@@ -958,6 +1535,7 @@ public class TreeView extends Container {
 //	native static final protected int gtk_tree_view_get_visible_rect(int treeView);
 //	native static final protected void gtk_tree_view_widget_to_tree_coords(int treeView, int wx, int wy, int[] tx, int[] ty);
 //	native static final protected void gtk_tree_view_tree_to_widget_coords(int treeView, int tx, int ty, int[] wx, int[] wy);
+//  native static final protected void gtk_tree_view_get_visible_range(Handle treeView, Handle startPath, Handle endPath);
 //	native static final protected void gtk_tree_view_enable_model_drag_source(int treeView, int buttonMask, int[] targets, int action);
 //	native static final protected void gtk_tree_view_enable_model_drag_dest(int treeView, int[] targets, int action);
 //	native static final protected void gtk_tree_view_unset_rows_drag_source(int treeView);
