@@ -14,10 +14,10 @@ import java.util.Vector;
 import org.gnu.glib.EventMap;
 import org.gnu.glib.EventType;
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
 import org.gnu.gtk.event.TreeViewColumnEvent;
 import org.gnu.gtk.event.TreeViewColumnListener;
+import org.gnu.javagnome.Handle;
 
 /**
  * The TreeViewColumn object is a visible column in a {@link TreeView} widget,
@@ -36,6 +36,17 @@ public class TreeViewColumn extends GObject {
 	protected TreeViewColumn(Handle handle) {
 		super(handle);
 	}
+    
+    protected static TreeViewColumn getTreeViewColumn(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        TreeViewColumn obj = (TreeViewColumn) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new TreeViewColumn(handle);
+        
+        return obj;
+    }
 
 	/**
 	 * Packs the cell into the beginning column. If expand is TRUE, then 
@@ -75,11 +86,7 @@ public class TreeViewColumn extends GObject {
 			return null;
 		CellRenderer[] cr = new CellRenderer[hndls.length];
 		for (int i = 0; i < hndls.length; i++) {
-			GObject obj = getGObjectFromHandle(hndls[i]);
-			if (null != obj)
-				cr[i] = (CellRenderer)obj;
-			else
-				cr[i] = new CellRenderer(hndls[i]);
+			cr[i] = CellRenderer.getCellRenderer(hndls[i]);
 		}
 		return cr;
 	}
@@ -232,12 +239,7 @@ public class TreeViewColumn extends GObject {
 	
 	public Widget getWidget() {
 	    Handle hndl = gtk_tree_view_column_get_widget(getHandle());
-		if (null == hndl)
-			return null;
-		GObject obj = getGObjectFromHandle(hndl);
-		if (null != obj)
-			return (Widget)obj;
-		return new Widget(hndl);
+		return Widget.getWidget(hndl);
 	}
 
 	/**
