@@ -16,8 +16,8 @@ import org.gnu.gdk.Point;
 import org.gnu.gdk.Screen;
 import org.gnu.gdk.WindowHints;
 import org.gnu.glib.GObject;
-import org.gnu.glib.Handle;
 import org.gnu.glib.Type;
+import org.gnu.javagnome.Handle;
 
 /**
  * A Window is a window that can be displayed on the screen. It also
@@ -26,6 +26,14 @@ import org.gnu.glib.Type;
  * the contained widget.
  */
 public class Window extends Bin {
+    
+    /**
+     * Constructs a top-level Window. This is the same as calling
+     * <code>new Window(WindowType.TOPLEVEL)</code>
+     */
+    public Window() {
+        this(WindowType.TOPLEVEL);
+    }
 
 	/**
 	 * Construct a new Window object.
@@ -45,6 +53,23 @@ public class Window extends Bin {
 	public Window(Handle handle) {
 	    super(handle);
 	}
+    
+    /**
+     * Create a new Window with a handle to a native resource
+     * returned from a call to the native libraries.
+     * 
+     * @param handle The handle that represents a pointer to a native resource.
+     */
+    public static Window getWindow(Handle handle) {
+        if (handle == null)
+            return null;
+        
+        Window obj = (Window) getGObjectFromHandle(handle);
+        if (obj == null)
+            obj = new Window(handle);
+        
+        return obj;
+    }
 	
 	/**
 	 * Set the title for this Window object
@@ -166,12 +191,7 @@ public class Window extends Bin {
 	 */
 	public Widget getFocusWidget() {
 	    Handle hndl = gtk_window_get_focus(getHandle());
-		if (null == hndl)
-			return null;
-		GObject obj = getGObjectFromHandle( hndl );
-		if (null != obj)
-			return (Widget)obj;
-		return new Widget(hndl);
+		return Widget.getWidget(hndl);
 	}
 	
 	/**
@@ -570,7 +590,7 @@ public class Window extends Bin {
 	    Handle[] hndls = gtk_window_list_toplevels();
 		Window[] wins = new Window[hndls.length];
 		for (int i = 0; i < hndls.length; i++) {
-			wins[i] = new Window(hndls[i]);
+			wins[i] = Window.getWindow(hndls[i]);
 		}
 		return wins;
 	}
@@ -623,6 +643,38 @@ public class Window extends Bin {
     public void setDefaultIconName( String name ) {
         gtk_window_set_default_icon_name( name );
     }
+	
+	/**
+	 * Fetches the transient parent of this window.
+	 * @return The transient parent for this window, or null if no transient
+	 * parent has been set.
+	 * @see #setTransientParent(Window)
+	 */
+	public Window getTransientParent() {
+		Handle handle = gtk_window_get_transient_for(getHandle());
+		return Window.getWindow(handle);
+	}
+	
+	/**
+	 * Dialog windows should be set transient for the main application window
+	 * they were spawned from. This allows window managers to, for example, keep
+	 * the dialog on top of the main window, or center the dialog over the main
+	 * window. On Windows, this method will put the child window on top of the
+	 * parent, the same way the window manager would have done in X.
+	 * 
+	 * @param parent
+	 *            Transient parent window.
+	 */
+	public void setTransientParent(Window parent) {
+		Handle parentHandle;
+		if (parent == null) {
+			parentHandle = getNullHandle();
+		}
+		else {
+			parentHandle = parent.getHandle();
+		}
+		gtk_window_set_transient_for(getHandle(), parentHandle);
+	}
 
 	/**
 	 * Retrieve the runtime type used by the GLib library.
