@@ -9,10 +9,9 @@
 
 #include <jni.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include <cairo.h>
+#include "gtk_java.h"
 
-#ifndef _Included_org_gnu_gdk_Display
-#define _Included_org_gnu_gdk_Display
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -34,10 +33,14 @@ JNIEXPORT jint JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1type
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1open
   (JNIEnv *env, jclass cls, jstring name)
 {
-	const gchar* n = (*env)->GetStringUTFChars(env, name, NULL);
-	jobject dis = getHandleFromPointer(env, gdk_display_open(n));
+	const gchar *n;
+	GdkDisplay *dis;
+	
+	n = (*env)->GetStringUTFChars(env, name, NULL);
+	dis = gdk_display_open(n);
+	
 	(*env)->ReleaseStringUTFChars(env, name, n);
-	return dis;
+	return getGObjectHandle(env, (GObject*)dis);
 }
 
 
@@ -48,7 +51,11 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1open
 JNIEXPORT jstring JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1name
   (JNIEnv *env, jclass cls, jobject display)
 {
-	const gchar* name = gdk_display_get_name((GdkDisplay*)getPointerFromHandle(env, display));
+	const gchar* name;
+	GdkDisplay *dis;
+	
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	name = gdk_display_get_name(dis);
 	return (*env)->NewStringUTF(env, name);
 }
 
@@ -59,7 +66,12 @@ JNIEXPORT jstring JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1name
 JNIEXPORT jint JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1n_1screens
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return (jint)gdk_display_get_n_screens((GdkDisplay*)getPointerFromHandle(env, display));
+	GdkDisplay *dis;
+	gint numScreens;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	numScreens = gdk_display_get_n_screens(dis);
+	return (jint)numScreens;
 }
 
 /*
@@ -69,8 +81,13 @@ JNIEXPORT jint JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1n_1screens
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1screen
   (JNIEnv *env, jclass cls, jobject display, jint screenNum)
 {
-	return getHandleFromPointer(env, gdk_display_get_screen(
-			(GdkDisplay*)getPointerFromHandle(env, display), screenNum));
+	GdkDisplay *dis;
+	GdkScreen *screen;
+	
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	screen = gdk_display_get_screen(dis, screenNum);
+	
+	return getGObjectHandle(env, (GObject*)screen);
 }
 
 /*
@@ -80,8 +97,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1screen
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1default_1screen
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return getHandleFromPointer(env, gdk_display_get_default_screen(
-			(GdkDisplay*)getPointerFromHandle(env, display)));
+	GdkDisplay *dis;
+	GdkScreen *screen;
+	
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	screen = gdk_display_get_default_screen(dis);
+	
+	return getGObjectHandle(env, (GObject*)screen);
 }
 
 /*
@@ -91,7 +113,10 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1default_1s
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1pointer_1ungrab
   (JNIEnv *env, jclass cls, jobject display, jint time)
 {
-	gdk_display_pointer_ungrab((GdkDisplay*)getPointerFromHandle(env, display), (guint32)time);
+	GdkDisplay *dis;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	gdk_display_pointer_ungrab(dis, (guint32)time);
 }
 
 /*
@@ -101,7 +126,10 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1pointer_1ungrab
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1keyboard_1ungrab
   (JNIEnv *env, jclass cls, jobject display, jint time)
 {
-	gdk_display_keyboard_ungrab((GdkDisplay*)getPointerFromHandle(env, display), (guint32)time);
+	GdkDisplay *dis;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	gdk_display_keyboard_ungrab(dis, (guint32)time);
 }
 
 /*
@@ -111,7 +139,12 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1keyboard_1ungrab
 JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1pointer_1is_1grabbed
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return (jboolean)gdk_display_pointer_is_grabbed((GdkDisplay*)getPointerFromHandle(env, display));
+	GdkDisplay *dis;
+	jboolean result;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	result =  (jboolean)gdk_display_pointer_is_grabbed(dis);
+	return result;
 }
 
 /*
@@ -121,7 +154,10 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1pointer_1is_1g
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1beep
   (JNIEnv *env, jclass cls, jobject display)
 {
-	gdk_display_beep((GdkDisplay*)getPointerFromHandle(env, display));
+	GdkDisplay *dis;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	gdk_display_beep(dis);
 }
 
 /*
@@ -131,7 +167,10 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1beep
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1sync
   (JNIEnv *env, jclass cls, jobject display)
 {
-	gdk_display_sync((GdkDisplay*)getPointerFromHandle(env, display));
+	GdkDisplay *dis;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	gdk_display_sync(dis);
 }
 
 /*
@@ -141,7 +180,10 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1sync
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1flush
   (JNIEnv *env, jclass cls, jobject display)
 {
-	gdk_display_flush((GdkDisplay*)getPointerFromHandle(env, display));
+	GdkDisplay *dis;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	gdk_display_flush(dis);
 }
 
 /*
@@ -151,7 +193,10 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1flush
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1close
   (JNIEnv *env, jclass cls, jobject display)
 {
-	gdk_display_close((GdkDisplay*)getPointerFromHandle(env, display));
+	GdkDisplay *dis;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	gdk_display_close(dis);
 }
 
 /*
@@ -161,7 +206,10 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1close
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gdk_Display_gdk_1display_1list_1devices
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return getList(env,gdk_display_list_devices((GdkDisplay*)display));
+	GdkDisplay *dis;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	return getList(env,gdk_display_list_devices(dis));
 }
 
 /*
@@ -171,8 +219,13 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gdk_Display_gdk_1display_1list_1devi
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1event
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return getHandleFromPointer(env, gdk_display_get_event(
-			(GdkDisplay*)getPointerFromHandle(env, display)));
+	GdkDisplay *dis;
+	GdkEvent *event;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	event = gdk_display_get_event(dis);
+	return getGBoxedHandle(env, event, GDK_TYPE_EVENT, 
+                               NULL, gdk_event_free);
 }
 
 /*
@@ -182,8 +235,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1event
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1peek_1event
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return getHandleFromPointer(env, gdk_display_peek_event(
-			(GdkDisplay*)getPointerFromHandle(env, display)));
+	GdkDisplay *dis;
+	GdkEvent *event;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	event = gdk_display_peek_event(dis);
+	return getGBoxedHandle(env, event, GDK_TYPE_EVENT,
+                               NULL, gdk_event_free);
 }
 
 /*
@@ -193,8 +251,12 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1peek_1event
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1put_1event
   (JNIEnv *env, jclass cls, jobject display, jobject event)
 {
-	gdk_display_put_event((GdkDisplay*)getPointerFromHandle(env, display),
-						  (GdkEvent*)getPointerFromHandle(env, event));
+	GdkDisplay *dis;
+	GdkEvent *evt;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	evt = (GdkEvent*)getPointerFromHandle(env, event);
+	gdk_display_put_event(dis, evt);
 }
 
 /*
@@ -204,7 +266,10 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1put_1event
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1default
   (JNIEnv *env,  jclass cls)
 {
-	return getHandleFromPointer(env, gdk_display_get_default());
+	GdkDisplay *dis;
+	
+	dis = gdk_display_get_default();
+	return getGObjectHandle(env, (GObject*)dis);
 }
 
 /*
@@ -214,8 +279,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1default
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1core_1pointer
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return getHandleFromPointer(env, gdk_display_get_core_pointer(
-			(GdkDisplay*)getPointerFromHandle(env, display)));
+	GdkDisplay *dis;
+	GdkDevice *pointer;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	pointer = gdk_display_get_core_pointer(dis);
+	
+	return getGObjectHandle(env, (GObject*)pointer);
 }
 
 /*
@@ -223,17 +293,25 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1core_1poin
  * Method:    gdk_display_get_pointer
  */
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1pointer
-  (JNIEnv *env, jclass cls, jobject display, jobject screen, jintArray xVal, jintArray yVal, jintArray mask)
+  (JNIEnv *env, jclass cls, jobject display, jobject screen, 
+  jintArray xVal, jintArray yVal, jintArray mask)
 {
-	GdkScreen* scr = g_new(GdkScreen, 1);
-	int* x = (int*)(*env)->GetIntArrayElements(env, xVal, NULL);
-	int* y = (int*)(*env)->GetIntArrayElements(env, yVal, NULL);
-	GdkModifierType* mod = (GdkModifierType*)(*env)->GetIntArrayElements(env, mask, NULL);
-	gdk_display_get_pointer((GdkDisplay*)getPointerFromHandle(env, display), &scr, x, y, mod);
+	GdkDisplay *dis;
+	GdkScreen* scr;
+	int* x;
+	int* y;
+	GdkModifierType* mod;
+	
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	scr = g_new(GdkScreen, 1);
+	x = (int*)(*env)->GetIntArrayElements(env, xVal, NULL);
+	y = (int*)(*env)->GetIntArrayElements(env, yVal, NULL);
+	mod = (GdkModifierType*)(*env)->GetIntArrayElements(env, mask, NULL);
+	gdk_display_get_pointer(dis, &scr, x, y, mod);
 	(*env)->ReleaseIntArrayElements(env, xVal, (jint*)x, 0);
 	(*env)->ReleaseIntArrayElements(env, yVal, (jint*)y, 0);
 	(*env)->ReleaseIntArrayElements(env, mask, (jint*)mod, 0);
-	updateHandle(env, screen, scr);
+	screen = getGObjectHandle(env, (GObject*)scr);
 }
 
 /*
@@ -243,8 +321,12 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1pointer
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1window_1at_1pointer
   (JNIEnv *env, jclass cls, jobject display, jint x, jint y)
 {
-	return getHandleFromPointer(env, gdk_display_get_window_at_pointer(
-			(GdkDisplay*)getPointerFromHandle(env, display), (gint *)&x, (gint *)&y));
+	GdkDisplay *dis;
+	GdkDevice *pointer;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	pointer = gdk_display_get_window_at_pointer(dis, (gint *)&x, (gint *)&y);
+	return getGObjectHandle(env, (GObject*)pointer);
 }
 
 /*
@@ -254,7 +336,12 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1window_1at
 JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1supports_1cursor_1alpha
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return (jboolean)gdk_display_supports_cursor_alpha((GdkDisplay*)getPointerFromHandle(env, display));
+	GdkDisplay *dis;
+	jboolean result;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	result = (jboolean)gdk_display_supports_cursor_alpha(dis);
+	return result;
 }
 
 /*
@@ -264,7 +351,12 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1supports_1curs
 JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1supports_1cursor_1color
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return (jboolean)gdk_display_supports_cursor_color((GdkDisplay*)getPointerFromHandle(env, display));
+	GdkDisplay *dis;
+	jboolean result;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	result = (jboolean)gdk_display_supports_cursor_color(dis);
+	return result;
 }
 
 /*
@@ -274,7 +366,12 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1supports_1curs
 JNIEXPORT jint JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1default_1cursor_1size
   (JNIEnv *env, jclass cls, jobject display)
 {
-	return (jint)gdk_display_get_default_cursor_size((GdkDisplay*)getPointerFromHandle(env, display));
+	GdkDisplay *dis;
+	jint result;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	result = (jint)gdk_display_get_default_cursor_size(dis);
+	return result;
 }
 
 /*
@@ -284,9 +381,14 @@ JNIEXPORT jint JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1default_1curs
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1maximum_1cursor_1size
   (JNIEnv *env, jclass cls, jobject display, jintArray width, jintArray height)
 {
-	int* w = (int*)(*env)->GetIntArrayElements(env, width, NULL);
-	int* h = (int*)(*env)->GetIntArrayElements(env, height, NULL);
-	gdk_display_get_maximal_cursor_size((GdkDisplay*)getPointerFromHandle(env, display), w, h);
+	GdkDisplay *dis;
+	guint *w, *h;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	w = (guint*)(*env)->GetIntArrayElements(env, width, NULL);
+	h = (guint*)(*env)->GetIntArrayElements(env, height, NULL);
+	
+	gdk_display_get_maximal_cursor_size(dis, w, h);
 	(*env)->ReleaseIntArrayElements(env, width, (jint*)w, 0);
 	(*env)->ReleaseIntArrayElements(env, height, (jint*)h, 0);
 }
@@ -298,8 +400,12 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1maximum_1curs
 JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1supports_1selection_1notification
   (JNIEnv *env, jclass cls, jobject display)
 {
-	GdkDisplay* display_g = (GdkDisplay*)getPointerFromHandle(env, display);
-	return (jboolean)gdk_display_supports_selection_notification(display_g);
+	GdkDisplay *dis;
+	jboolean result;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	result = (jboolean)gdk_display_supports_selection_notification(dis);
+	return result;
 }
 
 /*
@@ -309,9 +415,14 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1supports_1sele
 JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1request_1selection_1notification
   (JNIEnv *env, jclass cls, jobject display, jobject atom)
 {
-	GdkDisplay* display_g = (GdkDisplay*)getPointerFromHandle(env, display);
-	GdkAtom* atom_g = (GdkAtom*)getPointerFromHandle(env, atom);
-	return (jboolean)gdk_display_request_selection_notification(display_g, *atom_g);
+	GdkDisplay *dis;
+	GdkAtom *atom_g;
+	jboolean result;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	atom_g = (GdkAtom*)getPointerFromHandle(env, atom);
+	result = (jboolean)gdk_display_request_selection_notification(dis, *atom_g);
+	return result;
 }
 
 /*
@@ -321,23 +432,33 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1request_1selec
 JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Display_gdk_1display_1supports_1clipboard_1persistence
   (JNIEnv *env, jclass cls, jobject display)
 {
-	GdkDisplay* display_g = (GdkDisplay*)getPointerFromHandle(env, display);
-	return (jboolean)gdk_display_supports_clipboard_persistence(display_g);	
+	GdkDisplay *dis;
+	jboolean result;
+
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	result = (jboolean)gdk_display_supports_clipboard_persistence(dis);	
+	return result;
 }
 
 /*
  * Class:     org_gnu_gdk_Display
  * Method:    gdk_display_store_clipboard
- * Signature: (Lorg/gnu/glib/Handle;Lorg/gnu/glib/Handle;I[Lorg/gnu/glib/Handle;)V
+ * Signature: (Lorg/gnu/javagnome/Handle;Lorg/gnu/javagnome/Handle;I[Lorg/gnu/javagnome/Handle;)V
  */
 JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1store_1clipboard
-  (JNIEnv *env, jclass cls, jobject display, jobject clipboardWindow, jint time, jobjectArray targets)
+  (JNIEnv *env, jclass cls, jobject display, jobject clipboardWindow, jint time, 
+  jobjectArray targets)
 {
-	GdkDisplay* display_g = (GdkDisplay*)getPointerFromHandle(env, display);
-	GdkWindow* clipboardWindow_g = (GdkWindow*)getPointerFromHandle(env, clipboardWindow);
-	GdkAtom* targets_g = (GdkAtom*)getPointerArrayFromHandles(env, targets);
-	jint size = (*env)->GetArrayLength(env, targets);
-	gdk_display_store_clipboard(display_g, clipboardWindow_g, time, targets_g, size);
+	GdkDisplay *dis;
+	GdkWindow* clipboardWindow_g;
+	GdkAtom* targets_g;
+	jint size;
+	
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	clipboardWindow_g = (GdkWindow*)getPointerFromHandle(env, clipboardWindow);
+	targets_g = (GdkAtom*)getPointerArrayFromHandles(env, targets);
+	size = (*env)->GetArrayLength(env, targets);
+	gdk_display_store_clipboard(dis, clipboardWindow_g, time, targets_g, size);
 }
 
 /*
@@ -347,12 +468,56 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1store_1clipboard
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Display_gdk_1display_1get_1default_1group
   (JNIEnv *env, jclass cls, jobject display)
 {
-	GdkDisplay* display_g = (GdkDisplay*)getPointerFromHandle(env, display);
-	return getHandleFromPointer(env, gdk_display_get_default_group(display_g));
+	GdkDisplay *dis;
+	GdkWindow* group;
+	
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	group = gdk_display_get_default_group(dis);
+	return getGObjectHandle(env, (GObject*)group);
 }
 
+/*
+ * Class:     org_gnu_gdk_Display
+ * Method:    gdk_display_set_double_click_time
+ * Signature: (Lorg/gnu/javagnome/Handle;I)V
+ */
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1set_1double_1click_1time
+  (JNIEnv *env, jclass cls, jobject display, jint time)
+{
+	GdkDisplay *dis;
+	
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	gdk_display_set_double_click_time(dis, time);
+}
+
+/*
+ * Class:     org_gnu_gdk_Display
+ * Method:    gdk_display_set_double_click_distance
+ * Signature: (Lorg/gnu/javagnome/Handle;I)V
+ */
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1set_1double_1click_1distance
+  (JNIEnv *env, jclass cls, jobject display, jint distance)
+{
+	GdkDisplay *dis;
+	
+	dis = (GdkDisplay*)getPointerFromHandle(env, display);
+	gdk_display_set_double_click_distance(dis, distance);
+}
+
+
+/*
+ * Class:     org_gnu_gdk_Display
+ * Method:    gdk_display_warp_pointer
+ * Signature: (Lorg/gnu/javagnome/Handle;Lorg/gnu/javagnome/Handle;II)V
+ */
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Display_gdk_1display_1warp_1pointer
+  (JNIEnv *env, jclass cls, jobject display, jobject screen, jint x, jint y)
+{
+	GdkDisplay* display_g = (GdkDisplay*)getPointerFromHandle(env, display);
+	GdkScreen *screen_g = (GdkScreen*)getPointerFromHandle(env, screen);
+	gdk_display_warp_pointer(display_g, screen_g, x, y);	
+}
   
-  #ifdef __cplusplus
+#ifdef __cplusplus
 }
 #endif
-#endif
