@@ -9,18 +9,13 @@
 
 #include <jni.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
 {
 #endif
 
-gboolean GdkPangoAttrEmbossed_get_embossed (GdkPangoAttrEmbossed * cptr) 
-{
-    return cptr->embossed;
-}
-
 /*
  * Class:     org.gnu.gdk.PangoAttrEmbossed
  * Method:    getEmbossed
@@ -29,7 +24,7 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_PangoAttrEmbossed_getEmbossed (JNIEn
     cls, jobject obj) 
 {
     GdkPangoAttrEmbossed *obj_g = (GdkPangoAttrEmbossed *)getPointerFromHandle(env, obj);
-    return (jboolean) (GdkPangoAttrEmbossed_get_embossed (obj_g));
+    return (jboolean) obj_g->embossed;
 }
 
 /*
@@ -40,7 +35,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_PangoAttrEmbossed_gdk_1pango_1attr_1e
     JNIEnv *env, jclass cls, jboolean embossed) 
 {
     gboolean embossed_g = (gboolean) embossed;
-    return getHandleFromPointer(env, gdk_pango_attr_embossed_new (embossed_g));
+    return getStructHandle(env, gdk_pango_attr_embossed_new (embossed_g),
+                           NULL, pango_attribute_destroy);
 }
 
 #ifdef __cplusplus
