@@ -9,7 +9,7 @@
 
 #include <jni.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -35,8 +35,10 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_PixbufAnimation_gdk_1pixbuf_1animatio
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
     GError* error_g = NULL;
-    jobject result = getHandleFromPointer(env, 
-    		gdk_pixbuf_animation_new_from_file (filename_g, &error_g));
+    jobject result = 
+        getGObjectHandle(env, 
+                         gdk_pixbuf_animation_new_from_file (filename_g, 
+                                                             &error_g));
     if (NULL != error_g)
     	updateHandle(env, error, error_g);
     (*env)->ReleaseStringUTFChars(env, filename, filename_g);
@@ -73,10 +75,10 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_PixbufAnimation_gdk_1pixbuf_1animatio
   (JNIEnv *env, jclass cls, jobject animation, jlong sec, jlong usec)
 {
     GdkPixbufAnimation *animation_g = (GdkPixbufAnimation *)getPointerFromHandle(env, animation);
-	GTimeVal* timeVal = g_new(GTimeVal, 1);
-	timeVal->tv_sec = sec;
-	timeVal->tv_usec = usec;
-	return getHandleFromPointer(env, gdk_pixbuf_animation_get_iter(animation_g, timeVal));
+    GTimeVal* timeVal = g_new(GTimeVal, 1);
+    timeVal->tv_sec = sec;
+    timeVal->tv_usec = usec;
+    return getGObjectHandle(env, gdk_pixbuf_animation_get_iter(animation_g, timeVal));
 }
 
 /*
@@ -100,7 +102,7 @@ Java_org_gnu_gdk_PixbufAnimation_gdk_1pixbuf_1animation_1get_1static_1image (JNI
     jclass cls, jobject animation) 
 {
     GdkPixbufAnimation *animation_g = (GdkPixbufAnimation *)getPointerFromHandle(env, animation);
-    return getHandleFromPointer(env, gdk_pixbuf_animation_get_static_image (animation_g));
+    return getGObjectHandle(env, gdk_pixbuf_animation_get_static_image (animation_g));
 }
 
 
