@@ -9,7 +9,7 @@
 
 #include <jni.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -57,12 +57,12 @@ Java_org_gnu_gdk_PixbufAnimationIter_gdk_1pixbuf_1animation_1iter_1get_1delay_1t
  * Class:     org.gnu.gdk.PixbufAnimationIter
  * Method:    gdk_pixbuf_animation_iter_get_pixbuf
  */
-JNIEXPORT jint JNICALL 
+JNIEXPORT jobject JNICALL 
 Java_org_gnu_gdk_PixbufAnimationIter_gdk_1pixbuf_1animation_1iter_1get_1pixbuf (JNIEnv *env, 
     jclass cls, jobject iter) 
 {
     GdkPixbufAnimationIter *iter_g = (GdkPixbufAnimationIter *)getPointerFromHandle(env, iter);
-    return (jint)gdk_pixbuf_animation_iter_get_pixbuf (iter_g);
+    return getGObjectHandle(env,gdk_pixbuf_animation_iter_get_pixbuf (iter_g));
 }
 
 /*
