@@ -9,7 +9,7 @@
 
 #include <jni.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -40,7 +40,7 @@ void GdkRectangle_set_x (GdkRectangle * cptr, gint32 x)
  * Class:     org.gnu.gdk.Rectangle
  * Method:    setX
  */
-JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_setX (JNIEnv *env, jobject obj, jint x) 
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_setX (JNIEnv *env, jclass cls, jobject obj, jint x) 
 {
     GdkRectangle *obj_g = (GdkRectangle *)getPointerFromHandle(env, obj);
     gint32 x_g = (gint32) x;
@@ -71,7 +71,7 @@ void GdkRectangle_set_y (GdkRectangle * cptr, gint32 y)
  * Class:     org.gnu.gdk.Rectangle
  * Method:    setY
  */
-JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_setY (JNIEnv *env, jobject obj, jint y) 
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_setY (JNIEnv *env, jclass cls, jobject obj, jint y) 
 {
     GdkRectangle *obj_g = (GdkRectangle *)getPointerFromHandle(env, obj);
     gint32 y_g = (gint32) y;
@@ -102,7 +102,7 @@ void GdkRectangle_set_width (GdkRectangle * cptr, guint32 width)
  * Class:     org.gnu.gdk.Rectangle
  * Method:    setWidth
  */
-JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_setWidth (JNIEnv *env, jobject obj, jint width) 
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_setWidth (JNIEnv *env, jclass cls, jobject obj, jint width) 
 {
     GdkRectangle *obj_g = (GdkRectangle *)getPointerFromHandle(env, obj);
     guint32 width_g = (guint32) width;
@@ -133,7 +133,7 @@ void GdkRectangle_set_height (GdkRectangle * cptr, guint32 height)
  * Class:     org.gnu.gdk.Rectangle
  * Method:    setHeight
  */
-JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_setHeight (JNIEnv *env, jobject obj, jint height)
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_setHeight (JNIEnv *env, jclass cls, jobject obj, jint height)
 {
     GdkRectangle *obj_g = (GdkRectangle *)getPointerFromHandle(env, obj);
     guint32 height_g = (guint32) height;
@@ -171,6 +171,39 @@ JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_gdk_1rectangle_1union (JNIEnv
     	updateHandle(env, dest, dest_g);
 }
 
+/*
+ * Class:     org.gnu.gdk.Rectangle
+ * Method:    gdk_rectangle_new
+ */
+JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Rectangle_gdk_1rectangle_1new
+  (JNIEnv *env, jclass cls) 
+{
+    GdkRectangle *obj_g = (GdkRectangle *)g_new(GdkRectangle, 1);
+    return getGBoxedHandle(env, obj_g, gdk_rectangle_get_type(), NULL, 
+    		(GBoxedFreeFunc) g_free);
+}
+
+/*
+ * Class:     org.gnu.gdk.Rectangle
+ * Method:    gdk_rectangle_free
+ */
+JNIEXPORT void JNICALL Java_org_gnu_gdk_Rectangle_gdk_1rectangle_1free
+  (JNIEnv *env, jclass cls, jobject obj) 
+{
+    GdkRectangle *obj_g = (GdkRectangle *)getPointerFromHandle(env, obj);
+    g_free(obj_g);
+}
+
+/*
+ * Class:     org.gnu.gdk.Rectangle
+ * Method:    gdk_rectangle_get_type
+ */
+JNIEXPORT jint JNICALL Java_org_gnu_gdk_Rectangle_gdk_1rectangle_1get_1type
+  (JNIEnv *env, jclass cls) 
+{
+    return (jint)gdk_rectangle_get_type();
+}
+
 
 #ifdef __cplusplus
 }
