@@ -9,7 +9,8 @@
 
 #include <jni.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifndef _Included_org_gnu_gtk_FileChooserHelper
 #define _Included_org_gnu_gtk_FileChooserHelper
@@ -193,8 +194,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 	len = g_slist_length(list);
 	strClass = (*env)->FindClass(env, "java/lang/String");
 	array = (*env)->NewObjectArray(env, len, strClass, NULL);
-	for (index = 0; index < len; index++) {
-		item = g_slist_nth(list, index);
+	item = list;
+	for (index = 0; item != NULL; item = item->next, ++index) {
 		value = (gchar*)item->data;
 		str = (*env)->NewStringUTF(env, value);
 		(*env)->SetObjectArrayElement(env, array, index, str);
@@ -302,8 +303,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 	len = g_slist_length(list);
 	strClass = (*env)->FindClass(env, "java/lang/String");
 	array = (*env)->NewObjectArray(env, len, strClass, NULL);
-	for (index = 0; index < len; index++) {
-		item = g_slist_nth(list, index);
+	item = list;
+	for (index = 0; item != NULL; item = item->next, ++index) {
 		value = (gchar*)item->data;
 		str = (*env)->NewStringUTF(env, value);
 		(*env)->SetObjectArrayElement(env, array, index, str);
@@ -356,7 +357,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_
   (JNIEnv *env, jclass cls, jobject chooser)
 {
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-	return getHandleFromPointer(env, gtk_file_chooser_get_preview_widget(chooser_g));
+	return getGObjectHandle(env, (GObject *) gtk_file_chooser_get_preview_widget(chooser_g));
 }
 
 /*
@@ -422,7 +423,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_
   (JNIEnv *env, jclass cls, jobject chooser)
 {
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-	return getHandleFromPointer(env, gtk_file_chooser_get_extra_widget(chooser_g));
+	return getGObjectHandle(env, (GObject *) gtk_file_chooser_get_extra_widget(chooser_g));
 }
 
 /*
@@ -480,7 +481,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_
   (JNIEnv *env, jclass cls, jobject chooser)
 {
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-	return getHandleFromPointer(env, gtk_file_chooser_get_filter(chooser_g));
+	return getGObjectHandle(env, (GObject *) gtk_file_chooser_get_filter(chooser_g));
 }
 
 /*
@@ -488,16 +489,17 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_
  * Method:    gtk_file_chooser_add_shortcut_folder
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1add_1shortcut_1folder
-  (JNIEnv *env, jclass cls, jobject chooser, jstring folder, jintArray error)
+  (JNIEnv *env, jclass cls, jobject chooser, jstring folder, jobject error)
 {
-	gchar* str;
+	const gchar* str;
 	GError* err;
 	gboolean bool;
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-
+	
 	str = (gchar*)(*env)->GetStringUTFChars(env, folder, NULL);
-	err = (GError*)(*env)->GetIntArrayElements(env, error, NULL);
+	err = (GError*) getPointerFromHandle(env, error);
 	bool = gtk_file_chooser_add_shortcut_folder(chooser_g, str, &err);
+	updateHandle(env, error, err);
 	(*env)->ReleaseStringUTFChars(env, folder, str);
 	(*env)->ReleaseIntArrayElements(env, error, (jint*)err, 0);
 	return (jboolean)bool;
@@ -510,14 +512,15 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser
 JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1remove_1shortcut_1folder
   (JNIEnv *env, jclass cls, jobject chooser, jstring folder, jintArray error)
 {
-	gchar* str;
+	const gchar* str;
 	GError* err;
 	gboolean bool;
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-
+	
 	str = (gchar*)(*env)->GetStringUTFChars(env, folder, NULL);
-	err = (GError*)(*env)->GetIntArrayElements(env, error, NULL);
+	err = (GError*) getPointerFromHandle(env, error);
 	bool = gtk_file_chooser_remove_shortcut_folder(chooser_g, str, &err);
+	updateHandle(env, error, err);
 	(*env)->ReleaseStringUTFChars(env, folder, str);
 	(*env)->ReleaseIntArrayElements(env, error, (jint*)err, 0);
 	return (jboolean)bool;
@@ -546,8 +549,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 	len = g_slist_length(list);
 	strClass = (*env)->FindClass(env, "java/lang/String");
 	array = (*env)->NewObjectArray(env, len, strClass, NULL);
-	for (index = 0; index < len; index++) {
-		item = g_slist_nth(list, index);
+	item = list;
+	for (index = 0; item != NULL; item = item->next, ++index) {
 		value = (gchar*)item->data;
 		str = (*env)->NewStringUTF(env, value);
 		(*env)->SetObjectArrayElement(env, array, index, str);
@@ -562,14 +565,15 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1add_1shortcut_1folder_1uri
   (JNIEnv *env, jclass cls, jobject chooser, jstring folder, jintArray error)
 {
-	gchar* str;
+	const gchar* str;
 	GError* err;
 	gboolean bool;
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-
+	
 	str = (gchar*)(*env)->GetStringUTFChars(env, folder, NULL);
-	err = (GError*)(*env)->GetIntArrayElements(env, error, NULL);
+	err = (GError*) getPointerFromHandle(env, error);
 	bool = gtk_file_chooser_add_shortcut_folder_uri(chooser_g, str, &err);
+	updateHandle(env, error, err);
 	(*env)->ReleaseStringUTFChars(env, folder, str);
 	(*env)->ReleaseIntArrayElements(env, error, (jint*)err, 0);
 	return (jboolean)bool;
@@ -582,14 +586,15 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser
 JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1remove_1shortcut_1folder_1uri
   (JNIEnv *env, jclass cls, jobject chooser, jstring folder, jintArray error)
 {
-	gchar* str;
+	const gchar* str;
 	GError* err;
 	gboolean bool;
 	GtkFileChooser* chooser_g = (GtkFileChooser*)getPointerFromHandle(env, chooser);
-
+	
 	str = (gchar*)(*env)->GetStringUTFChars(env, folder, NULL);
-	err = (GError*)(*env)->GetIntArrayElements(env, error, NULL);
+	err = (GError*) getPointerFromHandle(env, error);
 	bool = gtk_file_chooser_remove_shortcut_folder_uri(chooser_g, str, &err);
+	updateHandle(env, error, err);
 	(*env)->ReleaseStringUTFChars(env, folder, str);
 	(*env)->ReleaseIntArrayElements(env, error, (jint*)err, 0);
 	return (jboolean)bool;
@@ -618,8 +623,8 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 	len = g_slist_length(list);
 	strClass = (*env)->FindClass(env, "java/lang/String");
 	array = (*env)->NewObjectArray(env, len, strClass, NULL);
-	for (index = 0; index < len; index++) {
-		item = g_slist_nth(list, index);
+	item = list;
+	for (index = 0; item != NULL; item = item->next, ++index) {
 		value = (gchar*)item->data;
 		str = (*env)->NewStringUTF(env, value);
 		(*env)->SetObjectArrayElement(env, array, index, str);
@@ -627,19 +632,27 @@ JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1cho
 	return array;
 }
 
+JNIEXPORT void JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1set_1show_1hidden
+(JNIEnv *env, jclass cls, jobject chooser, jboolean show_hidden)
+{
+    GtkFileChooser * chooser_g = (GtkFileChooser *)getPointerFromHandle(env, chooser);
+    gboolean show_hidden_g = (gboolean)show_hidden;
+    gtk_file_chooser_set_show_hidden(chooser_g, show_hidden_g);
+}
+
+JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1get_1show_1hidden
+(JNIEnv *env, jclass cls, jobject chooser)
+{
+    GtkFileChooser * chooser_g = (GtkFileChooser *)getPointerFromHandle(env, chooser);
+    return (jboolean)gtk_file_chooser_get_show_hidden(chooser_g);
+}
+
 JNIEXPORT jint JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1get_1type
 (JNIEnv *env, jclass cls)
 {
     return (jint)gtk_file_chooser_get_type();
 }
 
-JNIEXPORT jobject JNICALL Java_org_gnu_gtk_FileChooserHelper_gtk_1file_1chooser_1error_1quark
-(JNIEnv *env, jclass cls)
-{
-    GQuark quark = (GQuark)gtk_file_chooser_error_quark();
-    return getHandleFromPointer(env, (void *)&quark);
-}
-
 #ifdef __cplusplus
 }
 #endif
