@@ -10,7 +10,8 @@
 #include <jni.h>
 #include <sys/types.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -33,7 +34,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_TreeViewColumn_gtk_1tree_1view_1column_1
  */
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeViewColumn_gtk_1tree_1view_1column_1new (JNIEnv *env, jclass cls) 
 {
-    return getHandleFromPointer(env, gtk_tree_view_column_new ());
+    return getGObjectHandle(env, (GObject *) gtk_tree_view_column_new ());
 }
 
 /*
@@ -418,7 +419,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gtk_TreeViewColumn_gtk_1tree_1view_1colum
 {
     GtkTreeViewColumn *treeColumn_g = 
         (GtkTreeViewColumn *)getPointerFromHandle(env, treeColumn);
-    return getHandleFromPointer(env, gtk_tree_view_column_get_widget (treeColumn_g));
+    return getGObjectHandle(env, (GObject *) gtk_tree_view_column_get_widget (treeColumn_g));
 }
 
 /*
