@@ -10,7 +10,8 @@
 #include <jni.h>
 #include <sys/types.h>
 #include <gtk/gtk.h>
-#include "jg_jnu.h"
+#include <jg_jnu.h>
+#include "gtk_java.h"
 
 #ifdef __cplusplus
 extern "C" 
@@ -35,7 +36,7 @@ JNIEXPORT jint JNICALL Java_org_gnu_gtk_Window_gtk_1window_1get_1type (JNIEnv *e
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Window_gtk_1window_1new (JNIEnv *env, jclass cls, jint type) 
 {
     GtkWindowType type_g = (GtkWindowType) type;
-    return getHandleFromPointer(env, gtk_window_new (type_g));
+    return getGObjectHandle(env, (GObject *) gtk_window_new(type_g));
 }
 
 /*
@@ -172,7 +173,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Window_gtk_1window_1set_1focus (JNIEnv *
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Window_gtk_1window_1get_1focus (JNIEnv *env, jclass cls, jobject window) 
 {
     GtkWindow *window_g = (GtkWindow *)getPointerFromHandle(env, window);
-    return getHandleFromPointer(env, gtk_window_get_focus (window_g));
+    return getGObjectHandle(env, (GObject *) gtk_window_get_focus (window_g));
 }
 
 /*
@@ -219,7 +220,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Window_gtk_1window_1set_1transient_1for
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Window_gtk_1window_1get_1transient_1for (JNIEnv *env, jclass cls, jobject window) 
 {
     GtkWindow *window_g = (GtkWindow *)getPointerFromHandle(env, window);
-    return getHandleFromPointer(env, gtk_window_get_transient_for (window_g));
+    return getGObjectHandle(env, (GObject *) gtk_window_get_transient_for (window_g));
 }
 
 /*
@@ -431,7 +432,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Window_gtk_1window_1set_1icon_1list
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_Window_gtk_1window_1get_1icon_1list (JNIEnv *env, jclass cls, jobject window) 
 {
     GtkWindow *window_g = (GtkWindow *)getPointerFromHandle(env, window);
-    return getHandleArrayFromGList(env, gtk_window_get_icon_list (window_g));
+    return getGObjectHandlesFromGList(env, (GObject *) gtk_window_get_icon_list (window_g));
 }
 
 /*
@@ -454,7 +455,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Window_gtk_1window_1set_1icon (JNIEnv *e
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Window_gtk_1window_1get_1icon (JNIEnv *env, jclass cls, jobject window) 
 {
     GtkWindow *window_g = (GtkWindow *)getPointerFromHandle(env, window);
-    return getHandleFromPointer(env, gtk_window_get_icon (window_g));
+    return getGObjectHandle(env, (GObject *) gtk_window_get_icon (window_g));
 }
 
 /*
@@ -487,7 +488,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Window_gtk_1window_1set_1default_1icon_1
  * */
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_Window_gtk_1window_1get_1default_1icon_1list(JNIEnv *env, jclass cls)
 {
-    return getHandleArrayFromGList(env,gtk_window_get_default_icon_list());
+    return getGObjectHandlesFromGList(env,gtk_window_get_default_icon_list());
 }
 
 /*
@@ -520,7 +521,7 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gtk_Window_gtk_1window_1get_1modal (JNIE
  */ 
 JNIEXPORT jobjectArray JNICALL Java_org_gnu_gtk_Window_gtk_1window_1list_1toplevels (JNIEnv *env, jclass cls)
 {
-    return getHandleArrayFromGList(env,gtk_window_list_toplevels());
+    return getGObjectHandlesFromGList(env,gtk_window_list_toplevels());
 }
 
 /*
@@ -882,7 +883,7 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Window_gtk_1window_1set_1screen(JNIEnv *
 JNIEXPORT jobject JNICALL Java_org_gnu_gtk_Window_gtk_1window_1get_1screen(JNIEnv *env, jclass cls, jobject window)
 {
     GtkWindow *window_g = (GtkWindow *)getPointerFromHandle(env, window);
-    return getHandleFromPointer(env, gtk_window_get_screen(window_g));
+    return getGObjectHandle(env, (GObject *) gtk_window_get_screen(window_g));
 }
 
 /*
